@@ -1,25 +1,24 @@
+/*
+ This file is part of SLRN.
+
+ Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA. 
+*/
 #ifndef _SLRN_ART_H
 #define _SLRN_ART_H
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
-
 #ifndef SLRNPULL_CODE
 extern int slrn_select_article_mode (Slrn_Group_Type *, int, int);
 extern void slrn_init_article_mode (void);
@@ -32,6 +31,9 @@ extern char *Slrn_Art_Help_Line;
 extern char *Slrn_Followup_Custom_Headers;
 extern char *Slrn_Reply_Custom_Headers;
 
+extern int Slrn_Simulate_Graphic_Chars;
+
+extern int Slrn_Article_Window_Border;
 extern int Slrn_Startup_With_Article;
 extern int Slrn_Show_Thread_Subject;
 extern int Slrn_Query_Next_Article;
@@ -47,7 +49,11 @@ extern int Slrn_Reads_Per_Update;
 extern int Slrn_High_Score_Min;
 extern int Slrn_Low_Score_Max;
 extern int Slrn_Kill_Score_Max;
+extern int Slrn_Signature_Hidden;
+extern int Slrn_Pgp_Signature_Hidden;
+
 extern char *Slrn_X_Browser;
+
 extern char *Slrn_NonX_Browser;
 #if SLRN_HAS_SPOILERS
 extern int Slrn_Spoiler_Char;
@@ -56,8 +62,13 @@ extern int Slrn_Spoiler_Display_Mode;
 #if SLRN_HAS_TILDE_FEATURE
 extern int Slrn_Use_Tildes;
 #endif
+extern int Slrn_Emphasized_Text_Mode;
+extern int Slrn_Emphasized_Text_Mask;
 extern int Slrn_New_Subject_Breaks_Threads;
 extern int Slrn_Sig_Is_End_Of_Article;
+extern int Slrn_Del_Article_Upon_Read;
+extern int Slrn_Followup_Strip_Sig;
+extern int Slrn_Process_Verbatum_Marks;
 
 #endif				       /* NOT SLRNPULL_CODE */
 
@@ -93,7 +104,7 @@ typedef struct Slrn_Header_Type
    char *realname;		       /* NOT Null terminated */
    unsigned int realname_len;
    unsigned int tag_number;
-#define MAX_TREE_SIZE 24
+#define MAX_TREE_SIZE 32
    unsigned char tree[MAX_TREE_SIZE];
 #if SLRN_HAS_GROUPLENS
    int gl_rating;
@@ -109,27 +120,62 @@ extern Slrn_Header_Type *Slrn_First_Header;
 extern Slrn_Header_Type *Slrn_Current_Header;
 
 extern int slrn_goto_header (Slrn_Header_Type *, int);
+extern int slrn_locate_header_by_msgid (char *, int);
 
 
 typedef struct Slrn_Article_Line_Type
 {
    struct Slrn_Article_Line_Type *next, *prev;
    unsigned int flags;
-#define HEADER_LINE	1
-#define HIDDEN_LINE	2
-#define QUOTE_LINE	4
-#define SIGNATURE_LINE	8
-#define WRAPPED_LINE	0x10
-#if SLRN_HAS_SPOILERS
-# define SPOILER_LINE	0x20
-#endif
+#define LINE_TYPE_MASK		0x00FF
+#define HEADER_LINE		0x0001
+#define QUOTE_LINE		0x0002
+#define SIGNATURE_LINE		0x0004
+#define PGP_SIGNATURE_LINE	0x0008
+#define VERBATUM_LINE		0x0010
+
+#define LINE_ATTRIBUTES_MASK	0x0700
+#define WRAPPED_LINE		0x0100
+#define SPOILER_LINE		0x0200
+#define HIDDEN_LINE		0x0400
+
+#define QUOTES_LEVEL_MASK	0x1000
+   /* The last few bits are devoted to quote levels. */
+#define QUOTE_LEVEL_SHIFT 12	       /* do NOT change this */
+#define MAX_QUOTE_LEVELS   8
+#define QUOTE_LEVEL(flags) ((flags) >> QUOTE_LEVEL_SHIFT)
+
    char *buf;
 }
 Slrn_Article_Line_Type;
 
-extern Slrn_Header_Type *slrn_find_header_with_msgid (char *);
+typedef struct
+{
+   Slrn_Article_Line_Type *lines;
+   Slrn_Article_Line_Type *cline;      /* current line */
+   int is_modified;		       /* non-zero if different from server */
+   int is_wrapped;
+   /* Eventually *_hidden will be replaced by a bitmapped quantity */
+   int quotes_hidden;
+   int headers_hidden;
+   int signature_hidden;
+   int pgp_signature_hidden;
+#if SLRN_HAS_VERBATUM_MARKS
+   int verbatum_hidden;
+#endif
+   int needs_sync;		       /* non-zero if line number needs updated */
+}
+Slrn_Article_Type;
 
-extern Slrn_Article_Line_Type *Slrn_Article_Lines;
+extern Slrn_Article_Type *Slrn_Current_Article;
+
+extern Slrn_Header_Type *slrn_find_header_with_msgid (char *);
+extern int slrn_set_visible_headers (char *);
+
+#if SLRN_HAS_SLANG
+extern char *Slrn_Visible_Headers_String;
+extern int slrn_is_hidden_headers_mode (void);
+#endif
 
 extern SLRegexp_Type *Slrn_Ignore_Quote_Regexp[];
 
@@ -164,10 +210,44 @@ extern int slrn_edit_score (Slrn_Header_Type *, char *);
 
 int slrn_get_next_pagedn_action (void);
 
+extern int slrn_string_to_article (char *str);
+
 #if SLRN_HAS_SORT_BY_SCORE
 extern int Slrn_Display_Score;
 #endif
 
 extern int slrn_set_header_format (unsigned int, char *);
 
+extern void slrn_art_sync_article (Slrn_Article_Type *);
+extern int _slrn_art_unhide_quotes (Slrn_Article_Type *a);
+extern int _slrn_art_hide_quotes (Slrn_Article_Type *a, int level);
+#if SLRN_HAS_SPOILERS
+extern void slrn_art_mark_spoilers (Slrn_Article_Type *a);
+#endif
+extern void slrn_art_mark_quotes (Slrn_Article_Type *a);
+extern void slrn_art_mark_signature (Slrn_Article_Type *a);
+extern int _slrn_art_unwrap_article (Slrn_Article_Type *a);
+extern int _slrn_art_wrap_article (Slrn_Article_Type *a);
+extern void _slrn_art_skip_quoted_text (Slrn_Article_Type *a);
+extern int _slrn_art_skip_digest_forward (Slrn_Article_Type *a);
+extern char *slrn_art_extract_header (Slrn_Article_Type *a, char *hdr, unsigned int len);
+extern void _slrn_art_hide_headers (Slrn_Article_Type *a);
+extern void _slrn_art_unhide_headers (Slrn_Article_Type *a);
+extern int _slrn_art_unfold_header_lines (Slrn_Article_Type *a);
+
+extern void slrn_mark_header_lines (Slrn_Article_Type *a);
+extern void _slrn_art_hide_signature (Slrn_Article_Type *a);
+extern void _slrn_art_unhide_signature (Slrn_Article_Type *a);
+extern void _slrn_art_hide_pgp_signature (Slrn_Article_Type *a);
+extern void _slrn_art_unhide_pgp_signature (Slrn_Article_Type *a);
+extern void slrn_art_mark_pgp_signature (Slrn_Article_Type *a);
+
+#if SLRN_HAS_VERBATUM_MARKS
+extern void _slrn_art_hide_verbatum (Slrn_Article_Type *a);
+extern void _slrn_art_unhide_verbatum (Slrn_Article_Type *a);
+extern void slrn_art_mark_verbatum (Slrn_Article_Type *a);
+#endif
+
+/* These are in artmisc.c */
+
 #endif				       /* _SLRN_ART_H */
