@@ -1,23 +1,22 @@
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
+/*
+ This file is part of SLRN.
 
+ Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA. 
+*/
 #include "config.h"
 #include "slrnfeat.h"
 
@@ -40,6 +39,8 @@
 #include "score.h"
 #include "util.h"
 
+
+/* Returns 0 if score file not modified, -1 if error, or 1 if scores modified. */
 int slrn_edit_score (Slrn_Header_Type *h, char *newsgroup)
 {
    char ch = 'e';
@@ -53,8 +54,8 @@ int slrn_edit_score (Slrn_Header_Type *h, char *newsgroup)
    int score;
    FILE *fp;
    time_t myclock;
-   
-   
+   int file_modified = 0;
+
    if (Slrn_Score_File == NULL)
      {
 	slrn_error ("A Score file has not been specified.");
@@ -149,6 +150,8 @@ int slrn_edit_score (Slrn_Header_Type *h, char *newsgroup)
 	q = SLregexp_quote_string (newsgroup, qregexp, sizeof(qregexp));
 	if (q == NULL) q = newsgroup;
 	fprintf (fp, "%%\tNewsgroup: %s\n", q);
+	
+	file_modified = 1;
      }
    
    Slrn_Prefix_Arg_Ptr = NULL;
@@ -158,14 +161,22 @@ int slrn_edit_score (Slrn_Header_Type *h, char *newsgroup)
 
    if (ch == 'e')
      {
-	if (-1 == slrn_edit_file (Slrn_Editor_Score, file, linenum + 1))
-	  {
-	     slrn_error ("Error calling editor.");
-	     return -1;
-	  }
+	int status;
+	
+	status = slrn_edit_file (Slrn_Editor_Score, file, linenum + 1, 1);
+
+	if ((status == -2)	       /* unmodified */
+	    && (file_modified == 0))
+	  return 0;
+
+	if (status == -1)
+	  return -1;
+	
+	/* drop */
      }
    slrn_message_now ("Reloading score file ...");
-   (void) slrn_read_score_file (file);
-   slrn_message ("Exit group for changes to take effect.");
-   return 0;
+   if (-1 == slrn_read_score_file (file))
+     return -1;
+
+   return 1;
 }
