@@ -1,25 +1,23 @@
 /* -*- mode: C; mode: fold -*- */
+/*
+ This file is part of SLRN.
 
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
+ Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
 
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA. 
+*/
 #include "config.h"
 #include "slrnfeat.h"
 
@@ -173,11 +171,13 @@ void slrn_add_group_ranges (Slrn_Group_Type *g, int min, int max) /*{{{*/
 	r->min = min;
 	r->max = max;
 	
+#if 0
 	/* For this case, min should be 1 */
 	if (next == &g->range)
 	  {
 	     min = r->min = 1;
 	  }
+#endif
      }
    
    unread -= max - min + 1;
@@ -288,7 +288,7 @@ static int group_sync_group_with_server (Slrn_Group_Type *g, int *minp, int *max
    
    if (status != OK_GROUP)
      {
-	g->flags &= ~GROUP_UNSUBSCRIBED;
+	g->flags |= GROUP_UNSUBSCRIBED;
 	slrn_error ("This group appears to be bogus.");
 	return -1;
      }
@@ -336,9 +336,13 @@ static int group_sync_group_with_server (Slrn_Group_Type *g, int *minp, int *max
    r = &g->range;
    if (r->next != NULL)
      {
+#if 0
+	/* stesch@parsec.rhein-neckar.de (Stefan Scholl) suggests moving this
+	 * below to avoid posibility of n < 0.
+	 */
 	if (r->next->min <= r->min)
 	  r->next->min = 1;
-	
+#endif	
 	n = r->max;
 	while (r->next != NULL)
 	  {
@@ -348,6 +352,11 @@ static int group_sync_group_with_server (Slrn_Group_Type *g, int *minp, int *max
 	if (n < 0) n = 0;
 	if (n > max_available) n = max_available;
 	g->unread = n;
+#if 1
+	r = &g->range;
+	if (r->next->min <= r->min)
+	  r->next->min = 1;
+#endif
      }
    
    return 0;
@@ -559,6 +568,12 @@ static int add_group (char *name, unsigned int len, /*{{{*/
 	
 	slrn_add_group_ranges (Slrn_Group_Current_Group, min, max);
      }
+
+   if ((g->range.next != NULL)
+       && (g->range.next->min < g->unread)
+       && (g->range.next->min > 1))
+     g->unread -= g->range.next->min - 1;
+
    return 0;
 }
 
@@ -643,12 +658,17 @@ static Slrn_Group_Type *process_xgtitle_info (void) /*{{{*/
    char buf [NNTP_BUFFER_SIZE];
    Slrn_Group_Type *first = NULL, *save = Slrn_Group_Current_Group;
    
-   while (Slrn_Server_Obj->sv_read_line(buf, sizeof(buf)) != NULL)
+   while (1)
      {
 	char *b, ch;
 	unsigned int len;
 	Slrn_Group_Type *g;
-	
+	int status;
+
+	status = Slrn_Server_Obj->sv_read_line(buf, sizeof(buf));
+	if (status <= 0)
+	  break;
+
 	b = buf;
 	while (((ch = *b) != 0)
 	       && (ch != ' ') 
@@ -859,8 +879,20 @@ void slrn_select_next_group (void) /*{{{*/
 /*}}}*/
 void slrn_select_prev_group (void) /*{{{*/
 {
+   if (Slrn_Group_Current_Group == NULL)
+     return;
+
+   while ((SLang_Error == 0) && (1 == slrn_group_up_n (1)))
+     {
+       if (Slrn_Group_Current_Group->unread == 0)
+         continue;
+
+       if (0 == slrn_group_select_group ())
+         break;
+     }
 }
 
+
 /*}}}*/
 
 
@@ -1381,7 +1413,6 @@ static void select_group_cmd (void)
 
 void slrn_post_cmd (void) /*{{{*/
 {
-   int ret;
    char *name, *dist = NULL;	       /* "world"; */
    char group[256];
    char subj[256];
@@ -1407,11 +1438,7 @@ void slrn_post_cmd (void) /*{{{*/
    if (slrn_read_input ("Newsgroup", NULL, group, 1, 1) <= 0) return;
    *subj = 0; if (slrn_read_input ("Subject", NULL, subj, 1, 0) <= 0) return;
    
-   ret = slrn_post (group, subj, dist);
-   if (ret < 0)
-     {
-	slrn_error ("Posting failed.");
-     }
+   (void) slrn_post (group, subj, dist);
 }
 
 /*}}}*/
@@ -1810,10 +1837,21 @@ Note: This step may take some time if you have a slow connection!!!\n");
    
    num = 0;
    
-   while (NULL != Slrn_Server_Obj->sv_read_line (line, sizeof (line) - 1))
+   while (1)
      {
 	unsigned char *b, *bmax, *dsc, ch;
+	int status;
 	
+	status = Slrn_Server_Obj->sv_read_line (line, sizeof (line));
+	if (status == -1)
+	  {
+	     slrn_error ("Error reading line from server");
+	     slrn_fclose (fp);
+	     return;
+	  }
+	if (status == 0)
+	  break;
+
 	num = num % 25;
 	if (num == 0)
 	  {
@@ -1902,7 +1940,14 @@ int slrn_read_group_descriptions (void) /*{{{*/
 
 /*}}}*/
 
-void slrn_check_new_groups (int create_flag) /*{{{*/
+/* Map 1998 --> 98, 2003 --> 3, etc.... */
+static int rfc977_patchup_year (int year)
+{
+   return year - 100 * (year / 100);
+}
+
+   
+int slrn_check_new_groups (int create_flag) /*{{{*/
 {
    FILE *fp;
    time_t tloc;
@@ -1986,11 +2031,11 @@ void slrn_check_new_groups (int create_flag) /*{{{*/
 	     break;
 	     
 	   case ERR_FAULT:	     
-	     return;
+	     return -1;
 	     
 	   case ERR_COMMAND:
 	     slrn_message ("Server does not implement NEWGROUPS command.");
-	     return;
+	     return 0;
 
 	   default:
 	     slrn_message ("Server failed to return proper response to NEWGROUPS:\n%s\n",
@@ -1999,8 +2044,18 @@ void slrn_check_new_groups (int create_flag) /*{{{*/
 	  }
 	
 	num = 0;
-	while (NULL != Slrn_Server_Obj->sv_read_line (line, sizeof (line)))
+	
+	while (1)
 	  {
+	     int status = Slrn_Server_Obj->sv_read_line (line, sizeof (line));
+	     if (status == 0)
+	       break;
+	     if (status == -1)
+	       {
+		  slrn_error ("Read from server failed");
+		  return -1;
+	       }
+
 	     /* line contains new newsgroup name */
 	     add_unsubscribed_group (line);
 	     num++;
@@ -2027,22 +2082,29 @@ I will patch the file up for you.\n", file);
      {
 	slrn_exit_error ("Unable to open %s to record date.", line);
      }
-   
+
+   /* According to rfc977, the year must be in the form YY with the closest
+    * century specifying the rest of the year, i.e., 99 is 1999, 30 is 2030,
+    * etc.
+    */
 #if defined(VMS) || defined(__BEOS__)
    /* gmtime is broken on BEOS */
    tm_struct = localtime (&tloc);
+   tm_struct->tm_year = rfc977_patchup_year (tm_struct->tm_year + 1900);
    fprintf (fp, "NEWGROUPS %02d%02d%02d %02d%02d%02d",
             tm_struct->tm_year, 1 + tm_struct->tm_mon,
             tm_struct->tm_mday, tm_struct->tm_hour,
             tm_struct->tm_min, tm_struct->tm_sec);
 #else
    tm_struct = gmtime (&tloc);
+   tm_struct->tm_year = rfc977_patchup_year (tm_struct->tm_year + 1900);
    fprintf (fp, "NEWGROUPS %02d%02d%02d %02d%02d%02d GMT",
 	    tm_struct->tm_year, 1 + tm_struct->tm_mon,
 	    tm_struct->tm_mday, tm_struct->tm_hour,
 	    tm_struct->tm_min, tm_struct->tm_sec);
 #endif
    slrn_fclose (fp);
+   return 0;
 }
 
 /*}}}*/
@@ -2133,11 +2195,21 @@ int slrn_read_newsrc (int create_flag) /*{{{*/
 	if (OK_GROUPS != Slrn_Server_Obj->sv_list_active ())
 	  slrn_exit_error ("Server failed LIST ACTIVE.");
 
-	while (NULL != Slrn_Server_Obj->sv_read_line (line, sizeof(line)))
+	while (1)
 	  {
 	     unsigned int len;
 	     int min, max;
-	     
+	     int status;
+
+	     status = Slrn_Server_Obj->sv_read_line (line, sizeof(line));
+	     if (status == -1)
+	       {
+		  Slrn_Groups_Dirty = 0;
+		  slrn_exit_error ("Read from server failed");
+	       }
+	     if (status == 0)
+	       break;
+
 	     parse_active_line (line, &len, &min, &max);
 	     
 	     if (NULL == create_group_entry (line, len, min, max, 0, 0))
@@ -2271,6 +2343,7 @@ int slrn_read_newsrc (int create_flag) /*{{{*/
 	     find_line_num ();
 	  }
 	Slrn_Groups_Dirty = 1;
+	Slrn_Write_Newsrc_Flags = 0;
      }
    
    group_bob ();
