@@ -1,23 +1,22 @@
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
+/*
+ This file is part of SLRN.
 
+ Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA. 
+*/
 extern int slrn_select_group_mode (void);
 extern void slrn_group_quit (void);
 extern void slrn_select_next_group (void);
@@ -26,7 +25,7 @@ extern int slrn_group_select_group (void);
 extern int slrn_read_newsrc (int);
 extern int slrn_write_newsrc (void);
 extern void slrn_mark_article_as_read (char *, long);
-extern void slrn_check_new_groups (int);
+extern int slrn_check_new_groups (int);
 extern void slrn_init_group_mode (void);
 extern void slrn_get_group_descriptions (void);
 extern int slrn_read_group_descriptions (void);
