@@ -1,25 +1,23 @@
 /* -*- mode: C; mode: fold -*- */
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
+/*
+ This file is part of SLRN.
 
+ Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
 
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA. 
+*/
 #include "config.h"
 #include "slrnfeat.h"
 
@@ -574,13 +572,45 @@ static char *extract_article_header (char *h) /*{{{*/
 
 /*}}}*/
 
+
+static char *extract_current_article_header (char *h) /*{{{*/
+{
+   unsigned int len;
+   char buf[128];
+   
+   if (-1 == check_article_mode ())
+     return NULL;
+   if (NULL == Slrn_Current_Article)
+     {
+	slrn_error ("No article selected");
+	return NULL;
+     }
+
+   len = strlen (h);
+   if ((len + 3) <= sizeof (buf))
+     {
+	sprintf (buf, "%s: ", h);
+	h = slrn_art_extract_header (Slrn_Current_Article, buf, len + 2);
+     }
+   else h = NULL;
+
+   if (h == NULL) h = "";
+
+   return h;
+}
+
+/*}}}*/
+
 static char *article_as_string (void) /*{{{*/
 {
    unsigned int len;
    Slrn_Article_Line_Type *l;
    char *s, *s1;
    
-   l = Slrn_Article_Lines;
+   if (Slrn_Current_Article == NULL)
+     return "";
+
+   l = Slrn_Current_Article->lines;
 
    len = 0;
    while (l != NULL)
@@ -604,7 +634,7 @@ static char *article_as_string (void) /*{{{*/
 	return NULL;
      }
    
-   l = Slrn_Article_Lines;
+   l = Slrn_Current_Article->lines;
    s1 = s;
    
    while (l != NULL)
@@ -953,6 +983,23 @@ static void log_message (char *buf)
    fflush (fp);
 }
 
+static void message_now (char *s)
+{
+   slrn_message_now ("%s", s);
+}
+
+static void set_visible_headers (char *h)
+{
+   (void) slrn_set_visible_headers (h);
+}
+
+static char *get_visible_headers (void)
+{
+   if (Slrn_Visible_Headers_String == NULL)
+     return "";
+   return Slrn_Visible_Headers_String;
+}
+
 static void set_header_display_format (int *num, char *fmt)
 {
    (void) slrn_set_header_format (*num, fmt);
@@ -963,8 +1010,37 @@ static void print_file (char *file)
    (void) slrn_print_file (file);
 }
 
+static int locate_header_by_msgid (char *msgid, int *qs)
+{
+   if (-1 == slrn_locate_header_by_msgid (msgid, *qs))
+     return 0;
+   return 1;
+}
+
+static void replace_article_cmd (char *str)
+{
+   if ((-1 == check_article_mode ())
+       || (Slrn_Current_Header == NULL))
+     return;
+
+   (void) slrn_string_to_article (str);
+}
+
+static int is_article_visible (void)
+{
+   int x = slrn_is_article_visible ();
+   if ((x & 1) == 0)
+     return 0;
+   return x;
+}
+
 static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
 {
+   MAKE_INTRINSIC_0("headers_hidden_mode", slrn_is_hidden_headers_mode, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_S("replace_article", replace_article_cmd, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_S("message_now", message_now, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_S("set_visible_headers", set_visible_headers, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("get_visible_headers", get_visible_headers, SLANG_STRING_TYPE),
    MAKE_INTRINSIC_S("print_file", print_file, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("open_log_file", open_log_file, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("close_log_file", close_log_file, SLANG_VOID_TYPE),
@@ -972,7 +1048,7 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_IS("set_header_display_format", set_header_display_format, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("get_prefix_arg", get_prefix_arg, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("reset_prefix_arg", reset_prefix_arg, SLANG_VOID_TYPE),
-
+   MAKE_INTRINSIC_SI("locate_header_by_msgid", locate_header_by_msgid, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("article_as_string", article_as_string, SLANG_STRING_TYPE),
    MAKE_INTRINSIC_S("call", slrn_call_command, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("collapse_thread", collapse_thread, SLANG_VOID_TYPE),
@@ -980,6 +1056,7 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_0("current_newsgroup", current_group_name, SLANG_STRING_TYPE),
    MAKE_INTRINSIC_SSS("definekey", definekey, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("extract_article_header", extract_article_header, SLANG_STRING_TYPE),
+   MAKE_INTRINSIC_S("extract_displayed_article_header", extract_current_article_header, SLANG_STRING_TYPE),
    MAKE_INTRINSIC_0("get_group_flags", get_group_flags, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("get_grouplens_score", get_grouplens_score, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("get_header_flags", get_header_flags, SLANG_INT_TYPE),
@@ -1000,7 +1077,8 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_0("header_next_unread", header_next_unread, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("header_up", header_up, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("input_pending", input_pending, SLANG_INT_TYPE),
-   MAKE_INTRINSIC_0("is_article_visible", slrn_is_article_visible, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("_is_article_visible", slrn_is_article_visible, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("is_article_visible", is_article_visible, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("is_article_window_zoomed", slrn_is_article_win_zoomed, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("is_group_mode", is_group_mode, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("is_thread_collapsed", is_thread_collapsed, SLANG_INT_TYPE),
@@ -1038,7 +1116,7 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_SS("undefinekey", undefinekey, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_I("ungetkey", ungetkey, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("update", update, SLANG_VOID_TYPE),
-   SLANG_END_TABLE
+   MAKE_INTRINSIC_0(NULL, NULL, 0)
 };
 
 /*}}}*/
@@ -1054,7 +1132,8 @@ static SLang_Intrin_Var_Type Intrin_Vars [] =
    MAKE_VARIABLE("HEADER_TAGGED", &Interp_Header_Tagged, SLANG_INT_TYPE, 1),
    MAKE_VARIABLE("SCREEN_HEIGHT", &SLtt_Screen_Rows, SLANG_INT_TYPE, 1),
    MAKE_VARIABLE("SCREEN_WIDTH", &SLtt_Screen_Cols, SLANG_INT_TYPE, 1),
-   SLANG_END_TABLE
+
+   MAKE_VARIABLE(NULL, NULL, 0, 0)
 };
 
 
