@@ -1,23 +1,3 @@
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
-
 /* MIME handling routines.
  *
  * Author: Michael Elkins <elkins@aero.org>
@@ -77,21 +57,16 @@ int Slrn_Mime_Needs_Metamail;
 
 char *Slrn_Mime_Display_Charset;
 
-/* These are all supersets of US-ASCII */
+/* These are all supersets of US-ASCII.  Only the first N characters are 
+ * matched, where N is the length of the table entry.
+ */
 static char *Compatable_Charsets[] =
 {
    "US-ASCII",			       /* This MUST be zeroth element */
-     "ISO-8859-1",
-     "ISO-8859-2",
-     "ISO-8859-3",
-     "ISO-8859-4",
-     "ISO-8859-5",
-     "ISO-8859-6",
-     "ISO-8859-7",
-     "ISO-8859-8",
-     "ISO-8859-9",
-     "KOI8-R",
-     NULL
+   "ISO-8859-",
+   "iso-latin1",		       /* knews adds his one */
+   "KOI8-R",
+   NULL
 };
 
 static char *Char_Set;
@@ -121,12 +96,16 @@ static int Encoding_Method;
    || (((x) <= '9') && ((x) >= '0')))
 #endif
 
-static Slrn_Article_Line_Type *find_header_line (char *header)
+static Slrn_Article_Line_Type *find_header_line (Slrn_Article_Type *a, char *header)
 {
-   Slrn_Article_Line_Type *line = Slrn_Article_Lines;
+   Slrn_Article_Line_Type *line;
    unsigned char ch = (unsigned char) UPPER_CASE(*header);
    unsigned int len = strlen (header);
-   
+
+   if (a == NULL)
+     return NULL;
+   line = a->lines;
+
    while ((line != NULL) && (line->flags & HEADER_LINE))
      {
 	unsigned char ch1 = (unsigned char) *line->buf;
@@ -148,28 +127,35 @@ static char *find_compatable_charset (char *cs, unsigned int len)
    compat_charset = Compatable_Charsets;
    while (*compat_charset != NULL)
      {
-	if ((0 == slrn_case_strncmp ((unsigned char *) cs,
-				     (unsigned char *) *compat_charset,
-				     len))
-	    && (len == strlen(*compat_charset)))
-	  return *compat_charset;
+	unsigned int len1;
 	
+	len1 = strlen (*compat_charset);
+	if (len1 <= len) 
+	  {
+	     if (0 == slrn_case_strncmp ((unsigned char *) cs,
+					 (unsigned char *) *compat_charset,
+					 len1))
+	       return *compat_charset;
+	  }
 	compat_charset++;
      }
    return NULL;
 }
 
-static int parse_content_type_line (void)
+static int parse_content_type_line (Slrn_Article_Type *a)
 {
-   Slrn_Article_Line_Type *line;
    char *b;
+   Slrn_Article_Line_Type *line;
    
+   if (a == NULL)
+     return -1;
+   line = a->lines;
    /* Use default: text/plain; charset=us-ascii */
    Content_Type = CONTENT_TYPE_TEXT;
    Content_Subtype = CONTENT_SUBTYPE_PLAIN;
    Char_Set = Compatable_Charsets[0];
    
-   if (NULL == (line = find_header_line ("Content-Type:")))
+   if (NULL == (line = find_header_line (a, "Content-Type:")))
      return 0;
    
    b = slrn_skip_whitespace (line->buf + 13);
@@ -258,15 +244,18 @@ static int parse_content_type_line (void)
    return 0;
 }
 
-static int parse_content_transfer_encoding_line (void)
+static int parse_content_transfer_encoding_line (Slrn_Article_Type *a)
 {
    Slrn_Article_Line_Type *line;
    unsigned char *buf;
    
+   if (a == NULL)
+     return -1;
+
    Encoding_Method = ENCODED_7BIT;
-   line = find_header_line ("Content-Transfer-Encoding:");
+   line = find_header_line (a, "Content-Transfer-Encoding:");
    if (line == NULL) return 0;
-   
+
    buf = (unsigned char *) slrn_skip_whitespace (line->buf + 26);
    if (*buf == '"') buf++;
    
@@ -345,7 +334,11 @@ static char *decode_base64 (char *dest, char *src, char *srcmax)
    while (src + 3 < srcmax)
      {
 	*dest++ = (BASE64(src[0]) << 2) | (BASE64(src[1]) >> 4);
+	
+	if (src[2] == '=') break;
 	*dest++ = ((BASE64(src[1]) & 0xf) << 4) | (BASE64(src[2]) >> 2);
+	
+	if (src[3] == '=') break;
 	*dest++ = ((BASE64(src[2]) & 0x3) << 6) | BASE64(src[3]);
 	src += 4;
      }
@@ -404,7 +397,11 @@ int slrn_rfc1522_decode_string (char *s)
        
 	while ((ch = *s) != 0)
 	  {
-	     if ((ch == ' ') || (ch == '\t') || (ch == '\n'))
+	     /* Appararantly some programs do not convert ' ' to '_' in 
+	      * quoted printable.  Sigh.
+	      */
+	     if (((ch == ' ') && (method != 'Q'))
+		 || (ch == '\t') || (ch == '\n'))
 	       break;
 	     if ((ch == '?') && (s[1] == '='))
 	       break;
@@ -447,14 +444,20 @@ int slrn_rfc1522_decode_string (char *s)
 
 
 
-static void rfc1522_decode_headers (void)
+static void rfc1522_decode_headers (Slrn_Article_Type *a)
 {
-   Slrn_Article_Line_Type *line = Slrn_Article_Lines;
+   Slrn_Article_Line_Type *line;
    
+   if (a == NULL)
+     return;
+   
+   line = a->lines;
+
    while ((line != NULL) && (line->flags & HEADER_LINE))
      {
 	if (slrn_rfc1522_decode_string (line->buf))
 	  {
+	     a->is_modified = 1;
 	     Slrn_Mime_Was_Modified = 1;
 	  }
 	line = line->next;
@@ -490,7 +493,7 @@ static int merge_if_soft_linebreak (Slrn_Article_Line_Type *curr_line)
 	/* Remove the excess = character... */
 	b[len] = '\0';
 	
-	if (NULL == (b = (char *) SLREALLOC (b, 1 + len + strlen (next_line->buf))))
+	if (NULL == (b = (char *) SLrealloc (b, 1 + len + strlen (next_line->buf))))
 	  return -1;
 
 	curr_line->buf = b;
@@ -508,10 +511,15 @@ static int merge_if_soft_linebreak (Slrn_Article_Line_Type *curr_line)
    return 0;
 }
 
-static void decode_mime_quoted_printable (void)
+static void decode_mime_quoted_printable (Slrn_Article_Type *a)
 {
-   Slrn_Article_Line_Type *line = Slrn_Article_Lines;
+   Slrn_Article_Line_Type *line;
    
+   if (a == NULL)
+     return;
+   
+   line = a->lines;
+
    /* skip to body */
    while ((line != NULL) && (line->flags & HEADER_LINE))
      line = line->next;
@@ -536,6 +544,7 @@ static void decode_mime_quoted_printable (void)
 	if (b < line->buf + len)
 	  {
 	     *b = 0;
+	     a->is_modified = 1;
 	     Slrn_Mime_Was_Modified = 1;
 	  }
 	
@@ -552,20 +561,20 @@ void slrn_mime_article_init (void)
    Slrn_Mime_Needs_Metamail = 0;
 }
 
-void slrn_mime_process_article (void)
+void slrn_mime_process_article (Slrn_Article_Type *a)
 {
    if ((Slrn_Use_Mime == 0)
        || Slrn_Mime_Was_Parsed
-       || (Slrn_Article_Lines == NULL))
+       || (a == NULL))
      return;
-   
+
    Slrn_Mime_Was_Parsed = 1;	       /* or will be */
    
-   rfc1522_decode_headers ();
+   rfc1522_decode_headers (a);
 
-   if (NULL == find_header_line ("Mime-Version:")) return;
-   if ((-1 == parse_content_type_line ())
-       || (-1 == parse_content_transfer_encoding_line ()))
+   if (NULL == find_header_line (a, "Mime-Version:")) return;
+   if ((-1 == parse_content_type_line (a))
+       || (-1 == parse_content_transfer_encoding_line (a)))
      {
 	Slrn_Mime_Needs_Metamail = 1;
 	return;
@@ -584,7 +593,7 @@ void slrn_mime_process_article (void)
 	break;
 	
       case ENCODED_QUOTED:
-	decode_mime_quoted_printable ();
+	decode_mime_quoted_printable (a);
 	break;
 	
       default:
@@ -605,10 +614,14 @@ int slrn_mime_call_metamail (void)
    int init = Slrn_TT_Initialized;
    char cmd [2 * SLRN_MAX_PATH_LEN];
    char tempfile [SLRN_MAX_PATH_LEN];
-   Slrn_Article_Line_Type *ptr = Slrn_Article_Lines;
+   Slrn_Article_Line_Type *ptr;
    FILE *fp;
    char *tmp, *mm;
    
+   if (NULL == Slrn_Current_Article)
+     return -1;
+   ptr = Slrn_Current_Article->lines;
+     
    if ((Slrn_Use_Meta_Mail == 0)
        || Slrn_Batch
        || (slrn_get_yesno (1, "Process this MIME article with metamail") <= 0))
@@ -871,7 +884,7 @@ static int rfc1522_encode (unsigned char *d, unsigned int len,
 	     else if ((ch & 0x80) 
 		      || ((ch < 32) && (ch != '\n'))
 		      || IS_RFC850_SPECIAL(ch)
-		      || (ch == '=')  /* ? cannot occur as part of encoded word */
+		      || (ch == '=')  /* = cannot occur as part of encoded word */
 		      || (ch == '?')  /* ? cannot occur as part of encoded word */
 		      || (ch == '_'))  /* encode this since SPACE gets mapped to it */
 	       {
@@ -883,7 +896,7 @@ static int rfc1522_encode (unsigned char *d, unsigned int len,
 		    }
 		  sprintf ((char *)d, "=%02X", (int) ch);
 		  d += 3;
-		  if (ch & 0x80) hibit++;
+		  if (ch & 0x80) hibit--;
 	       }
 	     else *d++ = ch;
 	     p++;
