@@ -1,26 +1,24 @@
+/*
+ This file is part of SLRN.
+
+ Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA. 
+*/
 #ifndef _SLRN_MISC_H
 #define _SLRN_MISC_H
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
-
-
 #include <stdio.h>
 #include <slang.h>
 
@@ -55,7 +53,7 @@ typedef struct
 {
    char *realname;
    char *username;
-   char host[MAX_HOST_NAME_LEN];
+   char *hostname;
    char *replyto;
    char *org;
    char *followup_string;
@@ -68,7 +66,7 @@ Slrn_User_Info_Type;
 
 extern Slrn_User_Info_Type Slrn_User_Info;
 extern void slrn_get_user_info (void);
-extern int slrn_edit_file (char *, char *, unsigned int);
+extern int slrn_edit_file (char *, char *, unsigned int, int);
 extern  int slrn_mail_file (char *, int, unsigned int, char *, char *);
 
 extern void slrn_article_help (void);
@@ -105,8 +103,11 @@ extern char *Slrn_Editor_Post;
 extern char *Slrn_Editor_Score;
 extern char *Slrn_Editor_Mail;
 
+extern char *Slrn_Failed_Post_Filename;
+
 extern void slrn_push_keymap (SLKeyMap_List_Type *);
 extern void slrn_pop_keymap (void);
 extern SLKeyMap_List_Type *Slrn_Current_Keymap;
 
+extern int Slrn_Abort_Unmodified;
 #endif				       /* _SLRN_MISC_H */
