@@ -1,23 +1,22 @@
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
+/*
+ This file is part of SLRN.
 
+ Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA. 
+*/
 #include "config.h"
 #include "slrnfeat.h"
 
@@ -126,24 +125,24 @@ static int _nntp_initialize_server (void)
 }
 
 
-static char *_nntp_read_line (char *buf, unsigned int len)
+static int _nntp_read_line (char *buf, unsigned int len)
 {
    int status;
    
    status = nntp_read_line (NNTP_Server, buf, len);
    
    if (status == 1)
-     return buf;
+     return status;
    
    if (status == 0)
      {
 	_NNTP_Abort_On_Disconnection = 0;
-	return NULL;
+	return 0;
      }
 
    /* Read fail or user break.  Either way, we have to shut it down. */
    _nntp_connection_lost_hook (NNTP_Server);
-   return NULL;
+   return -1;
 }
 
 static int _nntp_head_cmd (int id, char *msgid, int *real_idp)
