@@ -1,23 +1,22 @@
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
+/*
+ This file is part of SLRN.
 
+ Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA. 
+*/
 #include "config.h"
 #include <stdio.h>
 #include <string.h>
@@ -71,6 +70,19 @@ static void _nntp_deallocate_nntp (NNTP_Type *s)
    memset ((char *) s, 0, sizeof (NNTP_Type));
 }
 
+/* Call connection-lost hook, unless it is ok to reconnect */
+static int check_connect_lost_hook (NNTP_Type *s)
+{
+   if (s->flags & NNTP_RECONNECT_OK)
+     return 0;
+
+   if (NNTP_Connection_Lost_Hook != NULL)
+     (*NNTP_Connection_Lost_Hook) (s);
+
+   return -1;
+}
+   
+
 int nntp_fgets_server (NNTP_Type *s, char *buf, unsigned int len)
 {
    if ((s == NULL) || (s->init_state <= 0))
@@ -81,6 +93,7 @@ int nntp_fgets_server (NNTP_Type *s, char *buf, unsigned int len)
    if (-1 == sltcp_fgets (s->tcp, buf, len))
      {
 	(void) nntp_disconnect_server (s);
+	(void) check_connect_lost_hook (s);
 	return -1;
      }
 
@@ -101,6 +114,7 @@ int nntp_fputs_server (NNTP_Type *s, char *buf)
    if (-1 == sltcp_fputs (s->tcp, buf))
      {
 	(void) nntp_disconnect_server (s);
+	(void) check_connect_lost_hook (s);
 	return -1;
      }
 
@@ -124,6 +138,7 @@ int nntp_write_server (NNTP_Type *s, char *buf, unsigned int n)
    if (n != sltcp_write (s->tcp, buf, n))
      {
 	(void) nntp_disconnect_server (s);
+	(void) check_connect_lost_hook (s);
 	return -1;
      }
 
@@ -153,6 +168,7 @@ int nntp_puts_server (NNTP_Type *s, char *buf)
        || (-1 == nntp_fputs_server (s, "\r\n"))
        || (-1 == sltcp_flush_output (s->tcp)))
      {
+	(void) check_connect_lost_hook (s);
 	nntp_disconnect_server (s);
 	return -1;
      }
@@ -166,6 +182,7 @@ static void _nntp_error_response (NNTP_Type *s, char *fmt)
    slrn_error ("Reason: %s", s->rspbuf);
 }
 
+
 static int _nntp_try_parse_timeout (char *str)
 {
    /* I know of only two timeout responses:
@@ -201,11 +218,19 @@ int nntp_get_server_response (NNTP_Type *s)
 
    status = atoi (s->rspbuf);
 
-   if (((status == ERR_FAULT) && (-1 != _nntp_try_parse_timeout (s->rspbuf)))
-       || (status == ERR_GOODBYE))
+   switch (status)
      {
+      case ERR_FAULT:
+	if (-1 == _nntp_try_parse_timeout (s->rspbuf))
+	  break;
+	/* Drop */
+	
+      case 0:			       /* invalid code */
+      case ERR_GOODBYE:
 	nntp_disconnect_server (s);
+	(void) check_connect_lost_hook (s);
 	status = -1;
+	break;
      }
 
    s->code = status;
@@ -215,12 +240,9 @@ int nntp_get_server_response (NNTP_Type *s)
 static int _nntp_reconnect (NNTP_Type *s)
 {
    nntp_disconnect_server (s);
-   if (0 == (s->flags & NNTP_RECONNECT_OK))
-     {
-	if (NNTP_Connection_Lost_Hook != NULL)
-	  (*NNTP_Connection_Lost_Hook) (s);
-	return -1;
-     }
+   
+   if (-1 == check_connect_lost_hook (s))
+     return -1;
 
    s->flags &= ~NNTP_RECONNECT_OK;
 
@@ -228,8 +250,7 @@ static int _nntp_reconnect (NNTP_Type *s)
 
    if (-1 == _nntp_connect_server (s))
      {
-	if (NNTP_Connection_Lost_Hook != NULL)
-	  (*NNTP_Connection_Lost_Hook) (s);
+	(void) check_connect_lost_hook (s);
 
 	s->flags |= NNTP_RECONNECT_OK;
 	return -1;
@@ -238,8 +259,7 @@ static int _nntp_reconnect (NNTP_Type *s)
    if ((s->group_name[0] != 0)
        && (-1 == nntp_server_vcmd (s, "GROUP %s", s->group_name)))
      {
-	if (NNTP_Connection_Lost_Hook != NULL)
-	  (*NNTP_Connection_Lost_Hook) (s);
+	(void) check_connect_lost_hook (s);
 
 	s->flags |= NNTP_RECONNECT_OK;
 	return -1;
@@ -420,20 +440,6 @@ static int _nntp_connect_server (NNTP_Type *s)
    return -1;
 }
 
-/* This mucks with the hostname buffer.  */
-static int _nntp_port_change (char *hostname, int *port)
-{
-   char *colon;
-
-   colon = slrn_strchr (hostname, ':');
-   if (colon == NULL) return -1;
-
-   *colon++ = 0;
-
-   *port = atoi (colon);
-   return 0;
-}
-
 #ifndef NNTPSERVER_FILE
 # define NNTPSERVER_FILE NULL
 #endif
@@ -502,6 +508,35 @@ char *nntp_get_server_name (void)
    return NULL;
 }
 
+
+/* In general, host has the form: "address:port".  If port is non-negative
+ * use its value despite the value coded in the hostname.
+ */
+static int _nntp_setup_host_and_port (char *host, int port, NNTP_Type *s)
+{
+   char *a, *b, *bmax;
+
+   a = host;
+   b = s->host;
+   bmax = b + NNTP_MAX_HOST_LEN;
+   while ((*a != 0) && (*a != ':') && (b < bmax))
+     *b++ = *a++;
+   *b = 0;
+
+   if (port <= 0)
+     {
+	if (*a == ':')
+	  port = atoi (a + 1);
+	else if (-1 == (port = sltcp_map_service_to_port ("nntp")))
+	  port = 119;
+     }
+
+   s->port = port;
+
+   return 0;
+}
+
+
 NNTP_Type *nntp_open_server (char *host, int port)
 {
    NNTP_Type *s;
@@ -518,21 +553,10 @@ NNTP_Type *nntp_open_server (char *host, int port)
 	  return NULL;
      }
 
-   (void) _nntp_port_change (host, &port);
-
-   if (port < 0)
-     {
-	port = sltcp_map_service_to_port ("nntp");
-	if (port == -1)
-	  port = 119;
-     }
-
    if (NULL == (s = nntp_allocate_nntp ()))
      return NULL;
 
-   strncpy (s->host, host, NNTP_MAX_HOST_LEN);
-   s->host [NNTP_MAX_HOST_LEN] = 0;
-   s->port = port;
+   (void) _nntp_setup_host_and_port (host, port, s);
 
    if (-1 == _nntp_connect_server (s))
      {
@@ -905,8 +929,115 @@ char *nntp_read_and_malloc (NNTP_Type *s)
      }
 
    if (status == 0)
-     return mbuf;
+     {
+	if (mbuf == NULL)
+	  mbuf = slrn_strmalloc ("", 0);
+	
+	return mbuf;
+     }
 
    slrn_free (mbuf);
    return NULL;
 }
+
+char *nntp_map_code_to_string (int code)
+{
+   switch (code)
+     {
+      case INF_HELP:	/* 100 */	
+	return "Help text on way";
+      case INF_AUTH:	/* 180 */	
+	return "Authorization capabilities";
+      case INF_DEBUG:	/* 199 */	
+	return "Debug output";
+      case OK_CANPOST:	/* 200 */	
+	return "Hello; you can post";
+      case OK_NOPOST:	/* 201 */	
+	return "Hello; you can't post";
+      case OK_SLAVE:	/* 202 */	
+	return "Slave status noted";
+      case OK_GOODBYE:	/* 205 */	
+	return "Closing connection";
+      case OK_GROUP:	/* 211 */	
+	return "Group selected";
+      case OK_GROUPS:	/* 215 */	
+	return "Newsgroups follow";
+      case OK_ARTICLE:	/* 220 */	
+	return "Article (head & body) follows";
+      case OK_HEAD:	/* 221 */
+	return "Head follows";
+      case OK_BODY:	/* 222 */
+	return "Body follows";
+      case OK_NOTEXT:	/* 223 */
+	return "No text sent -- stat, next, last";
+      case OK_NEWNEWS:	/* 230 */
+	return "New articles by message-id follow";
+      case OK_NEWGROUPS:/* 231 */
+	return "New newsgroups follow";
+      case OK_XFERED:	/* 235 */
+	return "Article transferred successfully";
+      case OK_POSTED:	/* 240 */
+	return "Article posted successfully";
+      case OK_AUTHSYS:	/* 280 */
+	return "Authorization system ok";
+      case OK_AUTH:	/* 281 */
+	return "Authorization (user/pass) ok";
+      case OK_XGTITLE:	/* 282 */
+	return "Ok, XGTITLE info follows";
+      case OK_XOVER:	/* 224 */
+	return "ok -- overview data follows";
+      case CONT_XFER:	/* 335 */
+	return "Continue to send article";
+      case CONT_POST:	/* 340 */
+	return "Continue to post article";
+      case NEED_AUTHINFO:/* 380 */
+	return "authorization is required";
+      case NEED_AUTHDATA:/* 381 */
+	return "<type> authorization data required";
+      case ERR_GOODBYE:	/* 400 */
+	return "Have to hang up for some reason";
+      case ERR_NOGROUP:	/* 411 */
+	return "No such newsgroup";
+      case ERR_NCING:	/* 412 */
+	return "Not currently in newsgroup";
+      case ERR_NOCRNT:	/* 420 */
+	return "No current article selected";
+      case ERR_NONEXT:	/* 421 */
+	return "No next article in this group";
+      case ERR_NOPREV:	/* 422 */
+	return "No previous article in this group";
+      case ERR_NOARTIG:	/* 423 */
+	return "No such article in this group";
+      case ERR_NOART:	/* 430 */
+	return "No such article at all";
+      case ERR_GOTIT:	/* 435 */
+	return "Already got that article, don't send";
+      case ERR_XFERFAIL:/* 436 */
+	return "Transfer failed";
+      case ERR_XFERRJCT:/* 437 */
+	return "Article rejected, don't resend";
+      case ERR_NOPOST:	/* 440 */
+	return "Posting not allowed";
+      case ERR_POSTFAIL:/* 441 */
+	return "Posting failed";
+      case ERR_NOAUTH:	/* 480 */
+	return "authorization required for command";
+      case ERR_AUTHSYS:	/* 481 */  /* ERR_XGTITLE has same code */
+	return "Authorization system invalid";
+      case ERR_AUTHREJ:	/* 482 */
+	return "Authorization data rejected";
+      case ERR_COMMAND:	/* 500 */
+	return "Command not recognized";
+      case ERR_CMDSYN:	/* 501 */
+	return "Command syntax error";
+      case ERR_ACCESS:	/* 502 */
+	return "Access to server denied";
+      case ERR_FAULT:	/* 503 */
+	return "Program fault, command not performed";
+      case ERR_AUTHBAD:	/* 580 */
+	return "Authorization Failed";
+     }
+
+   return "Unknown NNTP code";
+}
+
