@@ -1,24 +1,23 @@
-/* post an article */
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
+/*
+ This file is part of SLRN.
 
+ Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA. 
+*/
+/* post an article */
 #include "config.h"
 #include "slrnfeat.h"
 
@@ -54,42 +53,25 @@
 #include "util.h"
 #include "chmap.h"
 #include "menu.h"
+#include "version.h"
 
 #if SLRN_HAS_MIME
 # include "mime.h"
 #endif
 
+#define MAX_LINE_BUFLEN	2048
+
 char *Slrn_Courtesy_CC_Message = NULL;
 char *Slrn_Save_Posts_File;
 char *Slrn_Save_Replies_File;
 char *Slrn_Last_Message_Id;
 char *Slrn_Post_Custom_Headers;
+char *Slrn_Failed_Post_Filename;
+
 int Slrn_Reject_Long_Lines = 1;
 char *Slrn_Postpone_Dir;
 int Slrn_Generate_Message_Id = 1;
 
-#ifdef VMS
-# define SYSTEM_OS_NAME		"VMS"
-#else
-# ifdef __os2__
-#  define SYSTEM_OS_NAME	"OS/2"
-# else
-#  ifdef AMIGA
-#   define SYSTEM_OS_NAME	"Amiga"
-#  else
-#   ifdef __unix__
-#    define SYSTEM_OS_NAME       "UNIX"
-#   else
-#    if defined(__WIN32__) || defined(__NT__)
-#     define SYSTEM_OS_NAME	"Windows"
-#    else
-#     define SYSTEM_OS_NAME	"UNKNOWN"
-#    endif
-#   endif
-#  endif
-# endif
-#endif
-
 static int postpone_file (char *);
 
 #if SLRN_HAS_GEN_MSGID
@@ -147,7 +129,14 @@ static char *slrn_create_message_id (void)
    *b++ = '.';
    t = (unsigned char *) Slrn_User_Info.username;
    if (t != NULL)
-     while ((*t != 0) && (b < baseid + sizeof(baseid) - 1)) *b++ = *t++;
+     {
+	while ((*t != 0) && (b < baseid + sizeof(baseid) - 1))
+	  {
+	     if (*t == ' ')
+	       continue;
+	     *b++ = *t++;
+	  }
+     }
    *b = 0;
    
    return (char *) baseid;
@@ -200,10 +189,10 @@ static int slrn_cc_file (char *file, char *to, char *msgid)
    char outfile [SLRN_MAX_PATH_LEN];
 #endif
    FILE *pp, *fp;
-   char line[1024];
+   char line[MAX_LINE_BUFLEN];
    unsigned int cc_line = 0;
    unsigned int linenum;
-   char buf [1024];
+   char buf [MAX_LINE_BUFLEN];
    unsigned int nth;
    char *l;
 
@@ -278,9 +267,7 @@ static int slrn_cc_file (char *file, char *to, char *msgid)
    linenum = 0;
    
 #if SLRN_GEN_FROM_EMAIL_HEADER
-   fprintf (pp, "From: %s@%s (%s)\n",
-            Slrn_User_Info.username, Slrn_User_Info.host,
-            ((Slrn_User_Info.realname == NULL) ? "" : Slrn_User_Info.realname));
+   fprintf (pp, "From: %s\n", slrn_make_from_string ());
 #endif
    
    if (msgid != NULL)
@@ -345,7 +332,7 @@ static int slrn_cc_file (char *file, char *to, char *msgid)
 /* Returns -1 upon error, 0, if ok, 1 if needs repair, 2 is warning is issued */
 static int check_file_for_posting (char *file)
 {
-   char line[1024], *the_line;
+   char line[MAX_LINE_BUFLEN], *the_line;
    FILE *fp;
    unsigned int num;
    char *err;
@@ -522,7 +509,7 @@ int slrn_save_file_to_mail_file (char *file, char *save_file, char *msgid)
    FILE *infp, *outfp;
    time_t now;
    char save_post_file[SLRN_MAX_PATH_LEN];
-   char line [1024];
+   char line [MAX_LINE_BUFLEN];
 
    if ((save_file == NULL) || (*save_file == 0))
      return 0;
@@ -536,14 +523,16 @@ int slrn_save_file_to_mail_file (char *file, char *save_file, char *msgid)
    if (NULL == (outfp = slrn_open_home_file (save_file, "a",
 					     save_post_file, 1)))
      {
-	slrn_error ("Error saving to %s", save_post_file);
+	slrn_message_now ("***Warning: error saving to %s", save_post_file);
+	SLang_input_pending (30);
+	SLang_flush_input ();
 	slrn_fclose (infp);
-	return -1;
+	return 0;
      }
 	
    time (&now);
-   fprintf (outfp, "From %s@%s %s", Slrn_User_Info.username, Slrn_User_Info.host, ctime(&now));
-   fprintf (outfp, "From: %s@%s (%s)\n", Slrn_User_Info.username, Slrn_User_Info.host, Slrn_User_Info.realname);
+   fprintf (outfp, "From %s@%s %s", Slrn_User_Info.username, Slrn_User_Info.hostname, ctime(&now));
+   fprintf (outfp, "From: %s\n", slrn_make_from_string ());
    if (msgid != NULL) fprintf (outfp, "Message-Id: <slrn%s@%s>\n", msgid, Slrn_User_Info.posting_host);
 
    while (NULL != fgets (line, sizeof(line) - 1, infp))
@@ -626,11 +615,50 @@ static int post_references_header (char *line)
    return 0;
 }
 
+static int saved_failed_post (char *file)
+{
+   FILE *fp, *outfp;
+   char filebuf[SLRN_MAX_PATH_LEN];
+   char line[256];
+   char *save_file;
+
+   if (NULL == (save_file = Slrn_Failed_Post_Filename))
+     save_file = SLRN_FAILED_POST_FILE;
+   if (*save_file == 0)
+     return 0;
+
+   if (NULL == (fp = fopen (file, "r")))
+     return -1;
+
+   if (NULL == (outfp = slrn_open_home_file (save_file, "a",
+					     filebuf, 1)))
+     {
+	slrn_message_now ("***Warning: error writing to %s", filebuf);
+	fclose (fp);
+	return -1;
+     }
+
+   slrn_message ("Failed post saved in %s", filebuf);
+
+   (void) fputs ("\n\n", outfp);
+
+   while (NULL != (fgets (line, sizeof (line), fp)))
+     {
+	if (EOF == fputs (line, outfp))
+	  break;
+     }
+   
+   fclose (fp);
+   fclose (outfp);
+   return 0;
+}
+
 
 /* This function returns 1 if postponed, 0 upon sucess, -1 upon error */
 int slrn_post_file (char *file, char *to, int is_postponed)
 {
-   char line[1024], *linep;
+   char line[MAX_LINE_BUFLEN]; /* also used for MIME encoding of the realname */
+   char *linep;
    int len, header;
    FILE *fp;
    int rsp;
@@ -644,11 +672,14 @@ int slrn_post_file (char *file, char *to, int is_postponed)
    SLang_Name_Type *filter_hook;
 #endif
    int once_more;
-   
+   char *system_os_name;
+
 #if SLRN_HAS_SLANG
    filter_hook = SLang_get_function ("post_filter_hook");
 #endif
 
+   system_os_name = slrn_get_os_name ();
+
    try_again:
 
    perform_cc = 0;
@@ -736,11 +767,8 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 	     
 	   case 'e':
 	   case 'E':
-	     if (slrn_edit_file (Slrn_Editor_Post, file, 1) < 0) 
-	       {
-		  slrn_error ("User abort.");
-		  return -1;
-	       }
+	     if (slrn_edit_file (Slrn_Editor_Post, file, 1, 0) < 0)
+	       return -1;
 	     break;
 
 #if SLRN_HAS_SLANG
@@ -787,6 +815,7 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 	if (SLang_Error)
 	  {
 	     slrn_error ("post_file_hook returned error.  %s not posted.", file);
+	     (void) saved_failed_post (file);
 	     return -1;
 	  }
 
@@ -803,7 +832,8 @@ int slrn_post_file (char *file, char *to, int is_postponed)
    status = Slrn_Post_Obj->po_start ();
    if (status != CONT_POST)
      {
-	/* FIXME!! Put the file in a known place, e.g., dead.letter */
+	fclose (fp);
+	(void) saved_failed_post (file);
 	if (status != -1)
 	  slrn_error ("Posting not allowed.");
 	return -1;
@@ -811,8 +841,17 @@ int slrn_post_file (char *file, char *to, int is_postponed)
    
    if (Slrn_User_Info.username != NULL)
      Slrn_Post_Obj->po_printf ("Path: %s\n", Slrn_User_Info.username);
-   
-   Slrn_Post_Obj->po_printf ("From: %s@%s (%s)\n", Slrn_User_Info.username, Slrn_User_Info.host, Slrn_User_Info.realname);
+
+   /* line is not used until now, so we can use it as a buffer */
+   strncpy(line, slrn_make_from_string (), sizeof(line));
+   line[sizeof(line) - 1] = 0;
+
+#if SLRN_HAS_MIME
+   if (Slrn_Use_Mime)
+     slrn_mime_header_encode(line, sizeof(line));
+#endif
+
+   Slrn_Post_Obj->po_printf ("From: %s\n", line);
    /* if (Slrn_User_Info.posting_host != NULL)
      * Slrn_Post_Obj->po_printf ("X-Posting-Host: %s\n", Slrn_User_Info.posting_host); */
 
@@ -864,10 +903,10 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 #if SLRN_HAS_USER_AGENT
 		  Slrn_Post_Obj->po_printf ("User-Agent: slrn/%s (%s)\n\n", 
 					    Slrn_Version, 
-					    SYSTEM_OS_NAME);
+					    system_os_name);
 #else
 		  Slrn_Post_Obj->po_printf ("X-Newsreader: slrn (%s %s)\n\n", Slrn_Version,
-			       SYSTEM_OS_NAME);
+			       system_os_name);
 #endif
 		  header = 0;
 #if SLRN_HAS_MIME
@@ -920,22 +959,31 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 	linep = line + 1;
      }
    slrn_fclose (fp);
-   
+
    if (0 == Slrn_Post_Obj->po_end ())
      slrn_message ("Posting...done.");
    else
      {
-	if (Slrn_Batch) return -1;
+	if (Slrn_Batch) 
+	  {
+	     saved_failed_post (file);
+	     return -1;
+	  }
 
 	/* slrn_set_suspension (0); */
 	slrn_smg_refresh ();
 	sleep (2);
 	slrn_clear_message (); SLang_Error = 0;
 	rsp = slrn_get_response ("RrEeCc\007", "Select one: \001Repost, \001Edit, \001Cancel");
-	if (rsp == 7) return -1;
-	rsp |= 0x20;
-	if (rsp == 'c') return -1;
-	if ((rsp == 'e') && (slrn_edit_file (Slrn_Editor_Post, file, 1) < 0)) return -1;
+	
+	if ((rsp == 7)
+	    || (rsp == 'c') || (rsp == 'C')
+	    || (((rsp == 'e') || (rsp == 'E'))
+		&& (slrn_edit_file (Slrn_Editor_Post, file, 1, 0) < 0)))
+	  {
+	     (void) saved_failed_post (file);
+	     return -1;
+	  }
 	goto try_again;
      }
    
@@ -957,9 +1005,7 @@ int slrn_post (char *newsgroup, char *subj, char *dist)
    int ret;
    
    if (Slrn_Use_Tmpdir)
-     {
-	fp = slrn_open_tmpfile (file, "w");
-     }
+     fp = slrn_open_tmpfile (file, "w");
    else fp = slrn_open_home_file (SLRN_ARTICLE_FILENAME, "w", file, 0);
    
    if (fp == NULL)
@@ -989,10 +1035,8 @@ int slrn_post (char *newsgroup, char *subj, char *dist)
    slrn_add_signature (fp);
    slrn_fclose (fp);
    
-   if (slrn_edit_file (Slrn_Editor_Post, file, header_lines) >= 0)
-     {
-	ret = slrn_post_file (file, NULL, 0);
-     }
+   if (slrn_edit_file (Slrn_Editor_Post, file, header_lines, 1) >= 0)
+     ret = slrn_post_file (file, NULL, 0);
    else ret = -1;
    
    if (Slrn_Use_Tmpdir) (void) slrn_delete_file (file);
