@@ -1,23 +1,22 @@
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
+/*
+ This file is part of SLRN.
 
+ Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA. 
+*/
 #include "config.h"
 #ifndef SLRNPULL_CODE
 # include "slrnfeat.h"
@@ -469,7 +468,9 @@ int slrn_open_score (char *group_name)
    Group_Score_Type *gsc = Group_Score_Root;
    unsigned int n;
    int generic;
-   
+
+   free_group_chain ();
+
    if ((Slrn_Perform_Scoring == 0) || (Group_Score_Root == NULL))
      {
 	Slrn_Apply_Score = 0;
@@ -791,64 +792,107 @@ static unsigned long get_today (void)
    return yy * 10000 + mm * 100 + dd;
 }
 
-int slrn_read_score_file (char *file)
+typedef struct 
 {
    unsigned char group[256];
+   unsigned long today;
+   int score;
+   int start_new_group;
+   int gnt_not_flag;
+   int start_new_score;
+   int score_has_expired;
+   unsigned int pscore_flags;
+   PScore_Type *pst;
+   SLPreprocess_Type pt;
+}
+Score_Context_Type;
+
+static int read_score_file_internal (char *, Score_Context_Type *);
+
+static int handle_include_line (char *file, char *line, Score_Context_Type *c)
+{
+   char *dir, *f;
+   char buf [SLRN_MAX_PATH_LEN];
+   int status;
+
+   line = slrn_skip_whitespace (line);
+   
+   if (*line == 0)
+     return -1;
+   
+   (void) slrn_trim_string (line);
+
+#if defined(IBMPC_SYSTEM)
+   slrn_os2_convert_path (line);
+#endif
+
+   /* Make a filename which is relative to dir of current file.
+    * Note that slrn_dircat will properly handle absolute filenames.
+    */
+   
+   f = slrn_basename (file);
+   if (NULL == (dir = slrn_strnmalloc (file, (unsigned int) (f - file), 1)))
+     return -1;
+
+   if (-1 == slrn_dircat (dir, line, buf))
+     {
+	slrn_free (dir);
+	return -1;
+     }
+   
+   status = read_score_file_internal (buf, c);
+   slrn_free (dir);
+   return status;
+}
+	
+static int read_score_file_internal (char *file, Score_Context_Type *c)
+{
    char line[1024];
    FILE *fp;
-   SLPreprocess_Type pt;
-   int start_new_group = 1, not_flag, gnt_not_flag = 0;
    unsigned int linenum = 0;
-   int score = 0;
-   PScore_Type *pst = NULL;
-   unsigned long today;
-   int start_new_score = 0, score_has_expired = 0;
-   unsigned int pscore_flags = 0;
-   
-   today = get_today ();
-   
-   if (Group_Score_Root != NULL)
+
+   if (NULL == (fp = fopen (file, "r")))
      {
-	free_group_scores ();
+	slrn_error ("Unable to open score file %s", file);
+	return -1;
      }
-   
-   fp = fopen (file, "r");
-   
-   if (fp == NULL)
-     {
-	Slrn_Apply_Score = 0;
-	return 0;
-     }
-   
-   group[0] = '*';
-   group[1] = 0;
-   
-   (void) SLprep_open_prep (&pt);
+
    while (fgets (line, sizeof (line) - 1, fp))
      {
 	unsigned char *lp;
-	
+	int not_flag;
+
 	linenum++;
 
-	if (0 == SLprep_line_ok (line, &pt))
+	if (0 == SLprep_line_ok (line, &c->pt))
 	  continue;
 
 	lp = (unsigned char *) slrn_skip_whitespace (line);
 	if ((*lp == '#') || (*lp == '%') || (*lp <= ' ')) continue;
-	
+
+	if ((0 == slrn_case_strncmp (lp, (unsigned char *) "include", 7))
+	    && ((lp[7] == ' ') || (lp[7] == '\t')))
+	  {
+	     if (0 == handle_include_line (file, (char *)lp + 7, c))
+	       continue;
+	     
+	     score_error ("Error handling INCLUDE line", line, linenum, file);
+	     goto error_return;
+	  }
+
 	if (*lp == '[')
 	  {
 	     unsigned char *g, *gmax, ch;
-	     g = group;
-	     gmax = g + sizeof (group);
+	     g = c->group;
+	     gmax = g + sizeof (c->group);
 	     
 	     lp++;
 	     lp = (unsigned char *) slrn_skip_whitespace ((char *)lp);
 	     
-	     gnt_not_flag = 0;
+	     c->gnt_not_flag = 0;
 	     if (*lp == '~')
 	       {
-		  gnt_not_flag = 1;
+		  c->gnt_not_flag = 1;
 		  lp = (unsigned char *) slrn_skip_whitespace ((char *)lp + 1);
 	       }
 	     
@@ -862,38 +906,38 @@ int slrn_read_score_file (char *file)
 		  goto error_return;
 	       }
 	     *g = 0;
-	     start_new_group = 1;
-	     score_has_expired = 0;
-	     start_new_score = 0;
+	     c->start_new_group = 1;
+	     c->score_has_expired = 0;
+	     c->start_new_score = 0;
 	     continue;
 	  }
-	
+
 	if (!slrn_case_strncmp (lp, (unsigned char *)"Score:", 6))
 	  {
 	     unsigned char *lpp = lp + 6;
-	     pscore_flags = 0;
+	     c->pscore_flags = 0;
 	     if (*lpp == ':')
 	       {
 		  lpp++;
-		  pscore_flags |= SCORE_IS_OR_TYPE;
+		  c->pscore_flags |= SCORE_IS_OR_TYPE;
 	       }
 	     lpp = (unsigned char *) slrn_skip_whitespace ((char *) lpp);
 	     if (*lpp == '=')
 	       {
-		  pscore_flags |= RETURN_THIS_SCORE;
+		  c->pscore_flags |= RETURN_THIS_SCORE;
 		  lpp++;
 	       }
-	     score = atoi ((char *)lpp);
-	     start_new_score = 1;
+	     c->score = atoi ((char *)lpp);
+	     c->start_new_score = 1;
 	     continue;
 	  }
-	
-	if (start_new_score)
+
+	if (c->start_new_score)
 	  {
 	     if (!slrn_case_strncmp (lp, (unsigned char *) "Expires:", 8))
 	       {
 		  int ret;
-		  ret = has_score_expired (lp + 8, today);
+		  ret = has_score_expired (lp + 8, c->today);
 		  if (ret == -1)
 		    {
 		       score_error ("Expecting 'Expires: MM/DD/YYYY' or 'Expires: DD-MM-YYYY'",
@@ -904,28 +948,28 @@ int slrn_read_score_file (char *file)
 		    {
 		       slrn_message ("%s has expired score on line %d",
 				     file, linenum);
-		       start_new_score = 0;
-		       score_has_expired = 1;
+		       c->start_new_score = 0;
+		       c->score_has_expired = 1;
 		    }
-		  else score_has_expired = 0;
+		  else c->score_has_expired = 0;
 		  continue;
 	       }
+
 	     
-	     
-	     if (NULL == (pst = create_new_score (group, start_new_group, gnt_not_flag,
-						  score, pscore_flags)))
+	     if (NULL == (c->pst = create_new_score (c->group, c->start_new_group, c->gnt_not_flag,
+						     c->score, c->pscore_flags)))
 	       {
 		  score_error ("Bad group regular expression.", line, linenum, file);
 		  goto error_return;
 	       }
-	     start_new_group = 0;
-	     start_new_score = 0;
-	     score_has_expired = 0;
+	     c->start_new_group = 0;
+	     c->start_new_score = 0;
+	     c->score_has_expired = 0;
 	  }
 	
-	if (score_has_expired) continue;
+	if (c->score_has_expired) continue;
 	
-	if (pst == NULL)
+	if (c->pst == NULL)
 	  {
 	     score_error ("Expecting Score keyword.", line, linenum, file);
 	     goto error_return;
@@ -937,25 +981,24 @@ int slrn_read_score_file (char *file)
 	     lp++;
 	  }
 	else not_flag = 0;
-	
-	
+
 	/* Otherwise the line is a kill one */
 	if (!slrn_case_strncmp (lp, (unsigned char *)"Subject:", 8))
-	  add_group_regexp (pst, lp + 7, lp, SCORE_SUBJECT, not_flag);
+	  add_group_regexp (c->pst, lp + 7, lp, SCORE_SUBJECT, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"From:", 5))
-	  add_group_regexp (pst, lp + 4, lp, SCORE_FROM, not_flag);
+	  add_group_regexp (c->pst, lp + 4, lp, SCORE_FROM, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"Xref:", 5))
-	  add_group_regexp (pst, lp + 4, lp, SCORE_XREF, not_flag);
+	  add_group_regexp (c->pst, lp + 4, lp, SCORE_XREF, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"Newsgroup:", 10))
-	  add_group_regexp (pst, lp + 9, lp, SCORE_NEWSGROUP, not_flag);
+	  add_group_regexp (c->pst, lp + 9, lp, SCORE_NEWSGROUP, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"References:", 11))
-	  add_group_regexp (pst, lp + 10, lp, SCORE_REFERENCES, not_flag);
+	  add_group_regexp (c->pst, lp + 10, lp, SCORE_REFERENCES, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"Lines:", 6))
-	  add_group_regexp (pst, lp + 5, lp, SCORE_LINES, not_flag);
+	  add_group_regexp (c->pst, lp + 5, lp, SCORE_LINES, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"Date:", 5))
-	  add_group_regexp (pst, lp + 5, lp, SCORE_DATE, not_flag);
+	  add_group_regexp (c->pst, lp + 5, lp, SCORE_DATE, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"Message-Id:", 11))
-	  add_group_regexp (pst, lp + 10, lp, SCORE_MESSAGE_ID, not_flag);
+	  add_group_regexp (c->pst, lp + 10, lp, SCORE_MESSAGE_ID, not_flag);
 	else
 	  {
 	     unsigned char *lpp = lp;
@@ -966,18 +1009,49 @@ int slrn_read_score_file (char *file)
 		  goto error_return;
 	       }
 	     
-	     add_group_regexp (pst, lpp, lp, SCORE_GENERIC, not_flag);
+	     add_group_regexp (c->pst, lpp, lp, SCORE_GENERIC, not_flag);
 	  }
      }
-   
-   SLprep_close_prep (&pt);
+
    fclose (fp);
-   Slrn_Apply_Score = 1;
-   return 1;
+   return 0;
 
    error_return:
-   SLprep_close_prep (&pt);
    fclose (fp);
    return -1;
 }
 
+			
+int slrn_read_score_file (char *file)
+{
+   Score_Context_Type sc;
+   int status;
+
+   if (Group_Score_Root != NULL)
+     {
+	free_group_scores ();
+     }
+
+   sc.today = get_today ();
+   sc.score = 0;
+   sc.start_new_group = 1;
+   sc.gnt_not_flag = 0;
+   sc.start_new_score = 0;
+   sc.score_has_expired = 0;
+   sc.pscore_flags = 0;
+   sc.pst = NULL;
+   sc.group[0] = '*';
+   sc.group[1] = 0;
+
+   (void) SLprep_open_prep (&sc.pt);
+   status = read_score_file_internal (file, &sc);
+   SLprep_close_prep (&sc.pt);
+   
+   if (status == -1)
+     Slrn_Apply_Score = 0;
+   else
+     Slrn_Apply_Score = 1;
+
+   return status;
+}
+
