@@ -1,24 +1,23 @@
 /* -*- mode: C; mode: fold -*- */
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
+/*
+ This file is part of SLRN.
 
+ Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA. 
+*/
 #include "config.h"
 #include "slrnfeat.h"
 /*{{{ Include files */
@@ -36,6 +35,10 @@
 # include <unistd.h>
 #endif
 
+#ifdef HAVE_LOCALE_H
+# include <locale.h>
+#endif
+
 #ifdef VMS
 # include "vms.h"
 #else
@@ -527,9 +530,12 @@ static void slrn_set_screen_size (int sig) /*{{{*/
 
    if (sig)
      {
+#if SLANG_VERSION < 10306
 	SLsmg_reset_smg ();
 	SLsmg_init_smg ();
-	
+#else
+	SLsmg_reinit_smg ();
+#endif
 	slrn_redraw ();
      }
 }
@@ -584,6 +590,7 @@ int slrn_handle_interrupts (void)
 
 /*{{{ Screen Management and Terminal Init/Reset Functions */
 
+int Slrn_Use_Flow_Control = 0;
 static int init_tty (void) /*{{{*/
 {
    if (Slrn_TT_Initialized & SLRN_TTY_INIT)
@@ -594,7 +601,8 @@ static int init_tty (void) /*{{{*/
    if (Slrn_TT_Initialized == 0)
      init_display_signals (1);
 
-   SLang_init_tty (7, 1, 0);
+   if (-1 == SLang_init_tty (7, !Slrn_Use_Flow_Control, 0))
+     slrn_exit_error ("Unable to initialize the tty");
 
 #ifdef REAL_UNIX_SYSTEM
    SLang_getkey_intr_hook = slrn_handle_interrupts;
@@ -815,7 +823,7 @@ static void perform_cleanup (void)
    (void) slrn_reset_slang ();
 #endif
 
-#if SLRN_USE_SLTCP
+#if SLRN_USE_SLTCP && SLRN_HAS_NNTP_SUPPORT
    (void) sltcp_close_sltcp ();
 #endif
    lock_file (0);
@@ -901,115 +909,7 @@ For additional info use one of the forms:\n\
 
 /*}}}*/
 
-static char *make_slang_version (unsigned int v)
-{
-   unsigned int a, b, c;
-   static char buf[32];
-   
-   a = v/10000;
-   b = (v - a * 10000) / 100;
-   c = v - (a * 10000) - (b * 100);
 
-   sprintf (buf, "%u.%u.%u", a, b, c);
-   return buf;
-}
-
-
-static void version (void) /*{{{*/
-{
-   char *os;
-   
-#ifdef VMS
-   os = "VMS";
-#else
-# ifdef __os2__
-   os = "OS/2";
-# else
-#  ifdef AMIGA
-   os = "Amiga";
-#  else
-#   ifdef __unix__
-   os = "Unix";
-#   else
-#    if defined(__NT__) || defined(__WIN32__)
-   os = "Win32";
-#    else
-   os = "Unknown";
-#    endif
-#   endif
-#  endif
-# endif
-#endif
-   
-# if defined(__DATE__) && defined(__TIME__)
-   fprintf (stdout, "Slrn Version: %s (%s %s)\n", Slrn_Version, __DATE__, __TIME__);
-#else
-   fprintf (stdout, "Slrn Version: %s\n", Slrn_Version);
-#endif
-   fprintf (stdout, "S-Lang Library Version: %s\n", make_slang_version (SLang_Version));
-   if (SLANG_VERSION != SLang_Version)
-     {
-	fprintf (stdout, "\t** Note: This program was compiled against version %s.\n",
-		 make_slang_version (SLANG_VERSION));
-     }
-   fprintf (stdout, "OS-Type: %s\n", os);
-
-   fprintf (stdout, "\nslrn compiled with support for:");
-
-#if SLRN_HAS_CHARACTER_MAP
-   fputs ("\n\tcharacter set mapping", stdout);
-#endif
-   
-#if SLRN_HAS_NNTP_SUPPORT
-   fputs ("\n\tnntp", stdout);
-# if SLRN_USE_SLTCP
-   fputs (" (SLtcp socket code)", stdout);
-# endif
-# ifdef VMS
-#  ifdef MULTINET
-   fputs (" (Multinet)", stdout);
-#  endif
-#  ifdef UXC
-   fputs (" (UCX)", stdout);
-#  endif
-#  ifdef NETLIB
-   fputs (" (Netlib)", stdout);
-#  endif
-# endif
-#endif
-   
-#if SLRN_HAS_SPOOL_SUPPORT
-   fputs ("\n\tspool", stdout);
-#endif
-#if SLRN_HAS_INEWS_SUPPORT
-   fputs ("\n\tinews", stdout);
-#endif
-#if SLRN_HAS_PULL_SUPPORT
-   fputs ("\n\tslrnpull", stdout);
-#endif
-#if SLRN_HAS_GROUPLENS
-   fputs ("\n\tGroupLens", stdout);
-#endif
-#if SLRN_HAS_SLANG
-   fputs ("\n\tS-Lang Macros", stdout);
-#endif
-#if SLRN_HAS_MSGID_CACHE
-   fputs ("\n\tMsg-id Cache", stdout);
-#endif
-
-   fprintf (stdout, "\n\nDefault server object: %s\n",
-	    slrn_map_object_id_to_name (0, SLRN_DEFAULT_SERVER_OBJ));
-   
-   fprintf (stdout, "Default posting mechanism: %s\n",
-	    slrn_map_object_id_to_name (1, SLRN_DEFAULT_POST_OBJ));
-
-#if SLRN_HAS_CHARACTER_MAP
-   slrn_chmap_show_supported ();
-#endif
-   exit (0);
-}
-
-/*}}}*/
 static int parse_object_args (char *obj, char **argv, int argc) /*{{{*/
 {
    int num_parsed;
@@ -1068,6 +968,12 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
    char file [SLRN_MAX_PATH_LEN];
    char *init_file = SLRN_USER_SLRNRC_FILENAME;
 
+#ifdef HAVE_SETLOCALE
+# ifdef LC_CTYPE
+   (void) setlocale (LC_CTYPE, "");
+# endif
+#endif
+
    check_for_suspension ();
 #ifdef __unix__
    (void) umask (077);
@@ -1119,7 +1025,7 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 #endif
 	     else if (!strcmp ("create", argv_i)) create_flag = 1;
 	     else if (!strcmp ("version", argv_i))
-	       version ();
+	       slrn_show_version ();
 	     else usage (NULL);
 	  }
 	else if (!strcmp ("-create", argv_i)) create_flag = 1;
@@ -1176,7 +1082,7 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 	else if (use_color == -1) SLtt_Use_Ansi_Colors = 0;
      }
 
-#if SLRN_USE_SLTCP
+#if SLRN_USE_SLTCP && SLRN_HAS_NNTP_SUPPORT
    /* This is necessary to ensure that gethostname works on Win32 */
    if (-1 == sltcp_open_sltcp ())
      slrn_exit_error ("Error initializing SLtcp interface");
@@ -1252,7 +1158,8 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
     * has a username and a usable hostname.  Without those, we are not
     * starting up.
     */
-   if (0 == slrn_is_fqdn (Slrn_User_Info.host))
+   if ((NULL == Slrn_User_Info.hostname)
+       || (0 == slrn_is_fqdn (Slrn_User_Info.hostname)))
      {
 	slrn_exit_error ("\
 Unable to find a fully qualified hostname.  You will have to specify a\r\n\
@@ -1283,7 +1190,9 @@ cannot be constructed.  Try setting the USER environment variable.\r\n");
 #else
    Slrn_Newsrc_File = ".jnewsrc";
 #endif
-
+   slrn_message_now ("Using newsrc file %s for server %s.",
+		     Slrn_Newsrc_File, Slrn_Server_Obj->sv_name);
+   
    if (use_active) Slrn_List_Active_File = 1;
    if (use_mouse) Slrn_Use_Mouse = -1; /* -1 forces it. */
 
