@@ -1,29 +1,29 @@
+/*
+ This file is part of SLRN.
+
+ Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA. 
+*/
 #ifndef _SLRN_SLRN_H_
 #define _SLRN_SLRN_H_
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
-
 #include <slang.h>
 #include "version.h"
 
 extern int Slrn_TT_Initialized;
+extern int Slrn_Use_Flow_Control;
 extern void slrn_quit (int);
 
 extern int slrn_handle_interrupts (void);
@@ -93,7 +93,7 @@ extern int slrn_sys_system (char *);
 #define CURSOR_COLOR	6
 #define ARTICLE_COLOR	7
 #define TREE_COLOR	8
-#define QUOTE_COLOR	9
+#define PGP_SIGNATURE_COLOR 9
 #define SIGNATURE_COLOR 10
 #define THREAD_NUM_COLOR 11
 #define HIGH_SCORE_COLOR 12
@@ -109,4 +109,11 @@ extern int slrn_sys_system (char *);
 #define RESPONSE_CHAR_COLOR	22
 #define FRAME_COLOR		23
 #define SELECT_COLOR		24
+#define VERBATUM_COLOR		25
+
+#define BOLD_COLOR		30
+#define UNDERLINE_COLOR		31
+#define ITALICS_COLOR		32
+#define QUOTE_COLOR	40
+
 #endif				       /* _SLRN_SLRN_H_ */
