@@ -7,26 +7,6 @@
  * See also slrnfeat.h for customization of slrn features.
  */
 
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
-
 #define USE_DOMAIN_NAME 0
 #define MY_DOMAIN_NAME "foo.bar"
 
@@ -133,6 +113,15 @@
 # endif
 #endif
 
+/* Does VMS have these?  Do *all* VMS version have them? */
+#ifndef VMS
+# define HAVE_ISALPHA 1
+# define HAVE_ISSPACE 1
+# define HAVE_ISDIGIT 1
+# define HAVE_ISALNUM 1
+# define HAVE_ISPUNCT 1
+#endif
+
 #ifdef VMS
 # define USE_NOTEPAD_PRINT_CODE
 #endif
