@@ -1,5 +1,12 @@
 #ifndef _SLRN_FEATURES_H
 #define _SLRN_FEATURES_H
+/* Note!!!
+ * Lines beginning with '#' characters do NOT indicate that the line is
+ * commented out.  C does NOT use '#' as a comment character.  In this file, 
+ * '1' indicates that the feature will be turned on, and '0' indicates that
+ * the feature will be turned off.
+ */
+
 /* If you enable local SPOOL support, you must also enable NNTP support
  * or INEWS support.  Also, see below for filenames regarding the location
  * of the local spool and inews.  Currently only unix supports this.
@@ -20,8 +27,8 @@
 #define SLRN_HAS_MIME		1
 #define SLRN_HAS_SORT_BY_SCORE	1
 
+/* If non-zero, a User-Agent header will be generated instead of X-Newsreader. */
 #define SLRN_HAS_USER_AGENT	1
-/* If non-zero, a User-Agent string will be generated. */
 
 /* If non-zero '~' characters will be used at the end of an article.  This 
  * is what the vi editor does. */
@@ -32,8 +39,18 @@
  */
 #define SLRN_HAS_END_OF_THREAD	1
 
+/* If non-zero, text like *this* and _that_ will be highlighted. */
+#define SLRN_HAS_EMPHASIZED_TEXT	1
+
+/* If non-zero, slrn will treat specially marked lines of an article 
+ * as verbatum text.  It uses #v+ and #v- as markers of such text, e.g.,
+ * #v+
+ *    This line denotes verbatum text.
+ * #v-
+ */
+#define SLRN_HAS_VERBATUM_MARKS	1
+
 /* See README.GroupLens.  This only works if SLRN_USE_SLTCP is non-zero. 
- * This will not work under VMS because sltcp.c will not work with VMS.
  * Also make sure NNTP support is enabled.
  */
 #define SLRN_HAS_GROUPLENS	0
@@ -62,8 +79,8 @@
  */
 #define SLRN_HAS_UUDEVIEW	0
 
-/* If an article does not contain a references but it contains a reply-to
- * header, then the reply-to header might server as a references
+/* If an article does not contain a references but it contains an In-reply-to
+ * header, then the In-reply-to header might serve as a references
  * header. Set this to 1 to allow for that possibility.  Since this
  * only works if XOVER is not used, most NNTP users will not be
  * affected by it.  However, it could be a big win for slrnpull users
@@ -72,17 +89,15 @@
  */
 #define SLRN_HAS_FAKE_REFS	1
 
-/* The SLTCP code has not been ported to VMS due to the lack of a standardized
- * tcp-ip interface on VMS.  It should be ok on Unix and OS/2.
- */
+/* The sltcp interface is necessary for simultaneous tcp/ip connections. */
+#define SLRN_USE_SLTCP		1
+
 #ifndef VMS
 # define SLRN_HAS_DECODE	1
 # define SLRN_HAS_PIPING	1
-# define SLRN_USE_SLTCP 	1
 #else
 # define SLRN_HAS_PIPING	0
 # define SLRN_HAS_DECODE	0
-# define SLRN_USE_SLTCP		0
 #endif
 
 /* If non-zero, an rn style lock file will be created if .newsrc is the newsrc
@@ -164,6 +179,7 @@
 /* 
  *                                                            VMS filenames
  */
+# define SLRN_FAILED_POST_FILE		"slrn-failed-post.txt"
 # define SLRN_USER_SLRNRC_FILENAME	"slrn.rc"
 # define SLRN_LETTER_FILENAME		"slrn-letter.txt"
 # define SLRN_ARTICLE_FILENAME		"slrn-article.txt"
@@ -177,6 +193,7 @@
 /*
  *                                                             OS/2 filenames
  */
+#  define SLRN_FAILED_POST_FILE		"failpost.txt"
 #  define SLRN_USER_SLRNRC_FILENAME	"slrn.rc"
 #  define SLRN_LETTER_FILENAME		"letter.txt"
 #  define SLRN_ARTICLE_FILENAME		"article.txt"
@@ -191,6 +208,7 @@
 /* 
  *                                                             Unix filenames
  */
+#   define SLRN_FAILED_POST_FILE	"slrn-failed-post.txt"
 #   define SLRN_USER_SLRNRC_FILENAME	".slrnrc"
 #   define SLRN_LETTER_FILENAME	".letter"
 #   define SLRN_ARTICLE_FILENAME	".article"
