@@ -1,25 +1,23 @@
 /* -*- mode: C; mode: fold -*- */
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
+/*
+ This file is part of SLRN.
 
+ Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
 
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA. 
+*/
 #include "config.h"
 
 #define SLRNPULL_CODE
@@ -73,10 +71,13 @@
 #define SLRN_HAS_MSGID_CACHE 1
 
 #include "hash.c"
+
 /*}}}*/
 
 /*{{{ slrnpull global variables and structures */
 
+static char *Slrnpull_Version = SLRN_VERSION;
+
 #ifndef SLRNPULL_ROOT_DIR
 # define SLRNPULL_ROOT_DIR	"/var/spool/news/slrn"
 #endif
@@ -743,7 +744,7 @@ static int *listgroup_numbers (NNTP_Type *s, char *name, unsigned int *nump) /*{
    if (status != OK_GROUP)
      {
 	if (status == -1) log_error ("Read failed.");
-	log_error ("listgroup %s failed.", name);
+	log_error ("listgroup %s failed: %s", name, nntp_map_code_to_string (status));
 	return NULL;
      }
    
@@ -785,7 +786,9 @@ static int *listgroup_numbers (NNTP_Type *s, char *name, unsigned int *nump) /*{
 
 static unsigned int Num_Duplicates;
 
-static int *list_server_numbers (NNTP_Type *s, Active_Group_Type *g, unsigned int *nump) /*{{{*/
+static int *list_server_numbers (NNTP_Type *s, Active_Group_Type *g,
+				 int server_min, int server_max,
+				 unsigned int *nump) /*{{{*/
 {
    char *name; 
    unsigned int min, max;
@@ -803,18 +806,28 @@ static int *list_server_numbers (NNTP_Type *s, Active_Group_Type *g, unsigned in
    /* Server has XHDR.  Good. */
    min = g->min;
    max = g->max + 1;
-#if 0   
-   if (max >= min)
-     max++;
-#else
    if (max < min) max = min;
-#endif
 
-   status = nntp_server_vcmd (s, "XHDR Message-Id %d-", max);
+   /* If this is the first time this group has been visited, then do not grab
+    * all the message-ids when the user specifies a limit to grab.
+    */
+   if ((max <= 1) && (g->max_to_get != 0)
+       && ((unsigned int) server_max > g->max_to_get))
+     {
+	max = (unsigned int) server_max - g->max_to_get + 1;
+	if (server_max >= server_min)
+	  log_message ("The server may contain as many as %d articles in this group",
+		       server_max - server_min + 1);
+	log_message ("Only examining last %u articles since this is a new group", max);
+     }
+
+   status = nntp_server_vcmd (s, "XHDR Message-Id %u-", max);
    if (status == -1)
-     return NULL;			       /* server closed? */
-   
-   if (status == 224) status = OK_HEAD;/* Micro$soft broken server */
+     {
+	log_error ("Server failed to return proper code for XHDR.  The connection may be lost"); 
+	return NULL;			       /* server closed? */
+     }
+   if (status == 224) status = OK_HEAD;/* Micro$oft broken server */
    if (status != OK_HEAD)
      {
 	log_error ("Server failed XHDR command: %s", s->rspbuf);
@@ -918,7 +931,7 @@ static void print_time_stats (NNTP_Type *s, int do_log) /*{{{*/
 {
    char buf[512];
    time_t now;
-   unsigned int in;
+   unsigned long in;
    unsigned long elapsed_time;
    unsigned int hour, min, sec;
    
@@ -1302,7 +1315,7 @@ static int get_group_articles (NNTP_Type *s, Active_Group_Type *g,
      }
 
    Num_Duplicates = 0;
-   numbers = list_server_numbers (s, g, &num_numbers);
+   numbers = list_server_numbers (s, g, server_min, server_max, &num_numbers);
    if (Num_Duplicates)
      log_message ("%u duplicates removed leaving %u/%u.", 
 		  Num_Duplicates, num_numbers, num_numbers + Num_Duplicates);
@@ -1381,7 +1394,8 @@ static int pull_news (NNTP_Type *s) /*{{{*/
 	status = nntp_select_group (s, g->name, &min, &max);
 	if (status != OK_GROUP)
 	  {
-	     log_error ("Error selecting group %s.  Code = %d", g->name, status);
+	     log_error ("Error selecting group %s.  Code = %d: %s", g->name, 
+			status, nntp_map_code_to_string (status));
 
 	     if (status == -1)
 	       break;
@@ -1425,7 +1439,8 @@ static int post_file (NNTP_Type *s, char *file) /*{{{*/
    status = nntp_post_cmd (s);
    if (status != CONT_POST)
      {
-	log_error ("Server failed post cmd.  status = %d.", status);
+	log_error ("Server failed post cmd.  Code = %d: %s", 
+		   status, nntp_map_code_to_string (status));
 	fclose (fp);
 	return -1;
      }
@@ -1693,16 +1708,16 @@ static void close_servers (void) /*{{{*/
 
 static void init_signals (void);
 
-static void open_log_files (void) /*{{{*/
+static void open_log_files (char *logfile) /*{{{*/
 {
    char file [SLRN_MAX_PATH_LEN + 1];
    
    ELog_Fp = stderr;
    MLog_Fp = stdout;
    
-   if (-1 == slrn_dircat (SlrnPull_Dir, SLRNPULL_LOGFILE, file))
+   if (-1 == slrn_dircat (SlrnPull_Dir, logfile, file))
      return;
-   
+
    MLog_Fp = fopen (file, "a");
    if (MLog_Fp == NULL)
      {
@@ -1741,6 +1756,8 @@ static void usage (char *pgm) /*{{{*/
   --no-post         Do not post news\n\
   --new-groups      Get new groups\n\
   --version         Show version number\n\
+  --logfile FILE    Use FILE as the log file\n\
+  --debug FILE      Write dialogue with server to FILE\n\
 ",
 	      pgm, pgm);
    close_log_files ();
@@ -1751,7 +1768,7 @@ static void usage (char *pgm) /*{{{*/
 
 static void show_version (char *pgm)
 {
-   log_error ("%s version: %s\n", pgm, Slrn_Version);
+   log_error ("%s version: %s\n", pgm, Slrnpull_Version);
    close_log_files ();
    exit (0);
 }
@@ -1764,6 +1781,9 @@ static int read_score_file (void) /*{{{*/
    if (-1 == slrn_dircat (SlrnPull_Dir, SLRNPULL_SCORE_FILE, file))
      return -1;
    
+   if (1 != slrn_file_exists (file))
+     return 0;
+
    return slrn_read_score_file (file);
 }
 
@@ -1781,7 +1801,8 @@ int main (int argc, char **argv) /*{{{*/
    int post_mode;
    int check_new_groups = 0;
    char *dir;
-   
+   char *logfile;
+
    pgm = argv[0];
    argv++; argc--;
    
@@ -1791,6 +1812,7 @@ int main (int argc, char **argv) /*{{{*/
 
    MLog_Fp = stdout;
    ELog_Fp = stderr;
+   logfile = SLRNPULL_LOGFILE;
 
    dir = getenv ("SLRNPULL_ROOT");
    
@@ -1823,6 +1845,23 @@ int main (int argc, char **argv) /*{{{*/
 	  check_new_groups = 1;
 	else if (!strcmp (arg, "--version"))
 	  show_version (pgm);
+	else if (!strcmp (arg, "--logfile") && (argc > 0))
+	  {
+	     logfile = *argv;
+	     argv++; argc--;
+	  }
+	else if (!strcmp (arg, "--debug") && (argc > 0))
+	  {
+	     if (NULL == (NNTP_Debug_Fp = fopen (*argv, "w")))
+	       {
+		  fprintf (stderr, "Unable to create debug file %s\n", *argv);
+		  exit (1);
+	       }
+	     
+	     argv++;
+	     argc--;
+	  }
+
 	else usage (pgm);
      }
 
@@ -1835,7 +1874,7 @@ int main (int argc, char **argv) /*{{{*/
 	return 1;
      }
    
-   open_log_files ();
+   open_log_files (logfile);
 
    if (expire_mode)
      log_message ("slrnpull started in expire mode.");
@@ -1902,6 +1941,14 @@ int main (int argc, char **argv) /*{{{*/
 
 /*}}}*/
 
+
+/* Map 1998 --> 98, 2003 --> 3, etc.... */
+static int rfc977_patchup_year (int year)
+{
+   return year - 100 * (year / 100);
+}
+
+
 static int get_new_groups (NNTP_Type *s)
 {
    FILE *fp_time;
@@ -2031,12 +2078,14 @@ static int get_new_groups (NNTP_Type *s)
 #if defined(VMS) || defined(__BEOS__)
    /* gmtime is broken on BEOS */
    tm_struct = localtime (&tloc);
+   tm_struct->tm_year = rfc977_patchup_year (1900 + tm_struct->tm_year);
    fprintf (fp_time, "NEWGROUPS %02d%02d%02d %02d%02d%02d",
             tm_struct->tm_year, 1 + tm_struct->tm_mon,
             tm_struct->tm_mday, tm_struct->tm_hour,
             tm_struct->tm_min, tm_struct->tm_sec);
 #else
    tm_struct = gmtime (&tloc);
+   tm_struct->tm_year = rfc977_patchup_year (1900 + tm_struct->tm_year);
    fprintf (fp_time, "NEWGROUPS %02d%02d%02d %02d%02d%02d GMT",
 	    tm_struct->tm_year, 1 + tm_struct->tm_mon,
 	    tm_struct->tm_mday, tm_struct->tm_hour,
