@@ -1,26 +1,24 @@
+/*
+ This file is part of SLRN.
+
+ Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA. 
+*/
 #ifndef _SLTCP_H_LOADED_
 #define _SLTCP_H_LOADED_
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
-
-
 #if !defined(VMS) || defined(UCX)
 
 #include <stdarg.h>
@@ -37,8 +35,8 @@ typedef struct
 #define SLTCP_BUF_SIZE 8192
    unsigned char tcp_read_buf [SLTCP_BUF_SIZE];
    unsigned char tcp_write_buf [SLTCP_BUF_SIZE];
-   unsigned int bytes_out;
-   unsigned int bytes_in;
+   unsigned long bytes_out;
+   unsigned long bytes_in;
 }
 SLTCP_Type;
 
