@@ -1,23 +1,3 @@
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
-
 /* Local spool support for slrn added by Olly Betts <olly@mantis.co.uk> */
 #include "config.h"
 #include "slrnfeat.h"
@@ -71,15 +51,15 @@ static int spool_select_group (char *, int *, int *);
 static void spool_close_server (void);
 static int spool_has_cmd (char *);
 static int spool_initialize_server (void);
-static char *spool_read_line (char *, unsigned int );
+static int spool_read_line (char *, unsigned int );
 static int spool_xpat_cmd (char *, int, int, char *);
 static int spool_xgtitle_cmd (char *);
 static int spool_select_article (int, char *);
 static int spool_xhdr_command (char *, int, char *, unsigned int);
 static int spool_list_newsgroups (void);
 static int spool_list_active (void);
-static char *spool_read_xover (char *, unsigned int);
-static char *spool_read_xpat (char *, unsigned int);
+static int spool_read_xover (char *, unsigned int);
+static int spool_read_xpat (char *, unsigned int);
 static int spool_article_num_exists (int);
 
 static Slrn_Server_Obj_Type Spool_Server_Obj;
@@ -102,7 +82,7 @@ static int Spool_fhead=0; /* if non-0 we're emulating "HEAD" so stop on blank li
 static int Spool_fFakingActive=0; /* if non-0 we're doing funky stuff with MH folders */
 
 static int spool_fake_active( char *);
-static char *spool_fakeactive_read_line(char *, int);
+static int spool_fakeactive_read_line(char *, int);
 static int Spool_fakeactive_newsgroups=0;
 
 #if DEBUG_SPOOL
@@ -253,9 +233,8 @@ static int spool_nntp_xover (int min, int max)
    return OK_XOVER;
 }
 
-static char *spool_read_xover (char *the_buf, unsigned int len)
+static int spool_read_xover (char *the_buf, unsigned int len)
 {
-   char *p;
    long pos;
 
 #if DEBUG_SPOOL
@@ -263,12 +242,12 @@ static char *spool_read_xover (char *the_buf, unsigned int len)
 #endif
 
    if (Spool_Doing_XOver == 0)
-     return NULL;
+     return 0;
 
    if (Spool_XOver_Next > Spool_XOver_Max) 
      {
 	Spool_Doing_XOver = 0;
-	return NULL;
+	return 0;
      }
 
    while (1)
@@ -277,10 +256,10 @@ static char *spool_read_xover (char *the_buf, unsigned int len)
 
 	pos = ftell (Spool_fh_nov);
    
-	if (NULL == (p = fgets (the_buf, len, Spool_fh_nov)))
+	if (NULL == fgets (the_buf, len, Spool_fh_nov))
 	  {
 	     Spool_Doing_XOver = 0;
-	     return NULL;
+	     return 0;
 	  }
 	
 	buflen = strlen (the_buf);
@@ -288,12 +267,12 @@ static char *spool_read_xover (char *the_buf, unsigned int len)
 	  the_buf [buflen - 1] = 0;
    
 	/* check if we've reached the end of the requested range */
-	Spool_XOver_Next = atoi (p);
+	Spool_XOver_Next = atoi (the_buf);
 	if (Spool_XOver_Next > Spool_XOver_Max)
 	  {
 	     fseek (Spool_fh_nov, pos, SEEK_SET);
 	     Spool_Doing_XOver = 0;
-	     return NULL;
+	     return 0;
 	  }
 	
 	if (Slrn_Spool_Check_Up_On_Nov == 0)
@@ -319,7 +298,7 @@ static char *spool_read_xover (char *the_buf, unsigned int len)
      }
 
    Spool_XOver_Next++;
-   return p;
+   return 1;
 }
 
 static int spool_find_artnum_from_msgid (char *msgid)
@@ -352,7 +331,7 @@ static int spool_find_artnum_from_msgid (char *msgid)
    if (OK_XOVER != spool_nntp_xover (1, INT_MAX))
      return -1;
 
-   while (NULL != spool_read_xover (buf, sizeof(buf)))
+   while (1 == spool_read_xover (buf, sizeof(buf)))
      {
 	char *q;
 
@@ -726,6 +705,36 @@ static int spool_read_minmax_from_dp (Slrn_Dir_Type *dp, int *min, int *max)
    return 0;
 }
 
+static int spool_read_minmax_file (int *min, int *max, char *group_dir)
+{
+   char *file;
+   FILE *fp;
+   char buf[256];
+   int status;
+
+   file = slrn_spool_dircat (group_dir, ".minmax", 0);
+   if (file == NULL)
+     return -1;
+   
+   fp = fopen (file, "r");
+   if (fp == NULL)
+     {
+	SLFREE (file);
+	return -1;
+     }
+
+   status = 0;
+   if ((NULL == fgets (buf, sizeof (buf), fp))
+       || (2 != sscanf (buf, "%d %d", min, max)))
+     status = -1;
+   
+   fclose (fp);
+   SLFREE (file);
+   
+   return status;
+}
+
+
 /* Get the lowest and highest article numbers by the simple method
  * or looking at the files in the directory.
  * Returns 0 on success, -1 on failure
@@ -745,8 +754,11 @@ static int spool_read_minmax_from_dir (int *min, int *max, char *dir)
 
    if (-1 == spool_read_minmax_from_dp (dp, min, max))
      {
-	*min = 1;
-	*max = 0;
+	if (-1 == spool_read_minmax_file (min, max, dir))
+	  {
+	     *min = 1;
+	     *max = 0;
+	  }
      }
 
    (void) slrn_close_dir(dp);
@@ -911,21 +923,10 @@ static int spool_select_group (char *name, int *min, int *max)
 
    if ((-1 == spool_read_minmax_from_overview (name, min, max))
 #if SPOOL_ACTIVE_FOR_ART_RANGE
-       && (-1 == spool_read_minmax_from_active (name, min, max)))
+       && (-1 == spool_read_minmax_from_active (name, min, max))
 #endif
-       )
-     {
-	/* change directory to the spool directory.  The read_minmax_from_dir
-	 * function assumes it.
-	 */
-#if 0
-	if (chdir (Spool_Group))
-	  return ERR_NOGROUP;
-#endif
-	if (-1 == spool_read_minmax_from_dir (min, max, Spool_Group))
-	  return -1;
-	
-     }
+       && (-1 == spool_read_minmax_from_dir (min, max, Spool_Group)))
+     return -1;
 
    Spool_Max_Artnum = *max;
    Spool_Min_Artnum = *min;
@@ -987,7 +988,7 @@ static int spool_initialize_server (void)
    return 0;
 }
 
-static char *spool_read_line (char *line, unsigned int len)
+static int spool_read_line (char *line, unsigned int len)
 {
    if (Spool_Doing_XPat) return spool_read_xpat (line, len);
    
@@ -1000,14 +1001,14 @@ static char *spool_read_line (char *line, unsigned int len)
        || (Spool_fhead && (line[0]=='\n')))
      {
 	spool_fclose_local();
-	return NULL;
+	return 0;
      }
 
    len = strlen(line);
    if (len && (line [len - 1] == '\n'))
      line [len-1] = '\0';
-   
-   return line;
+
+   return 1;
 }
 
 typedef struct
@@ -1030,7 +1031,7 @@ static int spool_xpat_match (char *str, char *pat)
    return 0;
 }
 
-static char *spool_read_xpat (char *buf, unsigned int len)
+static int spool_read_xpat (char *buf, unsigned int len)
 {
    char tmpbuf [8129];
 
@@ -1058,18 +1059,18 @@ static char *spool_read_xpat (char *buf, unsigned int len)
 		  strncpy (buf + blen, tmpbuf, len - blen);
 		  buf[len - 1] = 0;
 
-		  return buf;
+		  return 1;
 	       }
 	  }
 	
 	Spool_XPat_Struct.rmin = Spool_XPat_Struct.rmax + 1;
-	return NULL;
+	return 0;
      }
    
    /* Not implemented Yet. */
    Spool_Doing_XPat = 0;
    spool_fclose_local ();
-   return NULL;
+   return 0;
 }
 
    
@@ -1176,7 +1177,7 @@ static int spool_xhdr_command (char *hdr, int num, char *buf, unsigned int bufle
    
    colon = strlen (hdr);
 
-   while (NULL != spool_read_line (tmpbuf, sizeof (tmpbuf)))
+   while (1 == spool_read_line (tmpbuf, sizeof (tmpbuf)))
      {
 	char *b;
 	if (slrn_case_strncmp ((unsigned char *) tmpbuf, (unsigned char *) hdr, colon)
@@ -1321,7 +1322,7 @@ static int spool_fake_active (char *path)
    return 0;
 }
 
-static char *spool_fakeactive_read_line(char *line, int len)
+static int spool_fakeactive_read_line(char *line, int len)
 {
    Slrn_Dirent_Type *ep;
    char *p;
@@ -1335,7 +1336,7 @@ static char *spool_fakeactive_read_line(char *line, int len)
      {
       /* we've reached the end of the road */
 	Spool_fFakingActive = 0;
-	return NULL;
+	return 0;
      }
 
    /* Scan through all the files, checking the ones with numbers for names */
@@ -1381,7 +1382,7 @@ static char *spool_fakeactive_read_line(char *line, int len)
 	sprintf(line,"%s %ld %ld y\n",Spool_nBuf,Spool_Head->hi,Spool_Head->lo); /* !HACK! check len */
      }
    spool_fake_active_out();
-   return line;
+   return 1;
 }
 
 static void spool_reset (void)
