@@ -1,24 +1,23 @@
 /* -*- mode: C; mode: fold; -*- */
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
+/*
+ This file is part of SLRN.
 
+ Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA. 
+*/
 /* Read startup .slrnrc file */
 
 #include "config.h"
@@ -79,6 +78,7 @@ static int grouplens_fun (int, SLcmd_Cmd_Table_Type *);
 static int interpret_fun (int, SLcmd_Cmd_Table_Type *);
 static int include_file_fun (int, SLcmd_Cmd_Table_Type *);
 static int set_header_format_fun (int, SLcmd_Cmd_Table_Type *);
+static int set_visible_headers_fun (int, SLcmd_Cmd_Table_Type *);
 
 /*}}}*/
 /*{{{ Static Global Variables */
@@ -109,7 +109,8 @@ static SLcmd_Cmd_Type Slrn_Startup_File_Cmds[] = /*{{{*/
      {include_file_fun, "include", "S"},
      {set_header_format_fun, "header_display_format", "IS"},
      {user_data_fun, "scorefile", "S"},
-   
+     {set_visible_headers_fun, "visible_headers", "S"},
+
    /* The following are considered obsolete */
      {user_data_fun, "username", "S"},
      {user_data_fun, "replyto", "S"},
@@ -304,6 +305,13 @@ static int grouplens_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 
 /*}}}*/
 
+
+static int set_visible_headers_fun (int argc, SLcmd_Cmd_Table_Type *table) 
+{
+   (void) argc;
+   return slrn_set_visible_headers (table->string_args[1]);
+}
+
 static int set_header_format_fun (int argc, SLcmd_Cmd_Table_Type *table) 
 {
    (void) argc;
@@ -323,6 +331,19 @@ Set_Int_Type;
 
 static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
 {
+     {"hide_signature",	&Slrn_Signature_Hidden},
+     {"hide_pgpsignature",	&Slrn_Pgp_Signature_Hidden},
+     {"emphasized_text_mask",	&Slrn_Emphasized_Text_Mask},
+     {"emphasized_text_mode",	&Slrn_Emphasized_Text_Mode},
+     {"process_verbatum_marks", &Slrn_Process_Verbatum_Marks},
+
+     {"use_flow_control",	&Slrn_Use_Flow_Control},
+     {"abort_unmodified_edits", &Slrn_Abort_Unmodified},
+     {"auto_mark_article_as_read", &Slrn_Del_Article_Upon_Read},
+     {"simulate_graphic_chars", &Slrn_Simulate_Graphic_Chars},
+#if 0 /* This does not work yet */
+     {"article_window_page_overlap", &Slrn_Article_Window_Border},
+#endif
      {"new_subject_breaks_threads",	&Slrn_New_Subject_Breaks_Threads},
      {"scroll_by_page", &Slrn_Scroll_By_Page},
      {"use_color", &SLtt_Use_Ansi_Colors},
@@ -364,6 +385,7 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
      {"min_high_score", &Slrn_High_Score_Min},
      {"max_low_score", &Slrn_Low_Score_Max},
      {"kill_score", &Slrn_Kill_Score_Max},
+     {"followup_strip_signature", &Slrn_Followup_Strip_Sig},
 #if !defined(IBMPC_SYSTEM)
      {"use_blink", &SLtt_Blink_Mode},
 #endif
@@ -428,6 +450,8 @@ Set_String_Type;
 
 static Set_String_Type String_Things_To_Set [] = /*{{{*/
 {
+     {"failed_posts_file", &Slrn_Failed_Post_Filename},
+     {"hostname", &Slrn_User_Info.hostname},
      {"art_help_line", &Slrn_Art_Help_Line},
      {"header_help_line", &Slrn_Header_Help_Line},
      {"group_help_line", &Slrn_Group_Help_Line},
@@ -676,35 +700,43 @@ typedef struct /*{{{*/
 Color_Handle_Type;
 
 /* default colors -- suitable for a color xterm */
-   
+
+#define DEF_BG		"black"
+#define DEF_FG		"lightgray"
+
 static Color_Handle_Type Color_Handles[] = /*{{{*/
 {
-     {"normal",		0,		"black",	"white", 0},
-     {"status",		STATUS_COLOR,	"yellow",	"blue", SLTT_REV_MASK},
-     {"menu",		MENU_COLOR,	"yellow",	"blue", SLTT_REV_MASK},
-     {"menu_press",	MENU_PRESS_COLOR,"blue",	"yellow", 0},
-     {"headers",	HEADER_COLOR,	"brightcyan",	"white", 0},
-     {"group",		GROUP_COLOR,	"blue",		"white", 0},
-     {"subject",	SUBJECT_COLOR,	"black",	"white", 0},
-     {"author",		AUTHOR_COLOR,	"magenta",	"white", 0},
-     {"error",		ERROR_COLOR,	"red",		"white", SLTT_BLINK_MASK},
-     {"cursor",		CURSOR_COLOR,	"brightgreen",	"white", SLTT_REV_MASK},
-     {"article",	ARTICLE_COLOR,	"blue",		"white", 0},
-     {"tree",		TREE_COLOR,	"red",		"white", 0},
-     {"quotes",		QUOTE_COLOR,	"red",		"white", 0},
-     {"signature",	SIGNATURE_COLOR,"red",		"white", 0},
-     {"thread_number",	THREAD_NUM_COLOR,"blue",	"white", SLTT_BOLD_MASK},
-     {"header_number",	HEADER_NUMBER_COLOR,"green",	"white", 0},
-     {"high_score",	HIGH_SCORE_COLOR,"red",		"white", SLTT_BOLD_MASK},
-     {"description",	GROUP_DESCR_COLOR,"magenta",	"white", 0},
-     {"grouplens_display",GROUPLENS_DISPLAY_COLOR,"blue","white", 0},
-     {"tilde",		SLRN_TILDE_COLOR,"green",	"white", SLTT_BOLD_MASK},
-     {"header_name",SLRN_HEADER_KEYWORD_COLOR,"green", "white", SLTT_BOLD_MASK},
-     {"response_char",RESPONSE_CHAR_COLOR,"green", "white", SLTT_BOLD_MASK},
-     {"box", BOX_COLOR,"black", "white", 0},
-     {"frame", FRAME_COLOR,"yellow", "blue", SLTT_REV_MASK},
-     {"selection", SELECT_COLOR,"yellow", "blue", SLTT_BOLD_MASK},
-   
+     {"article",	ARTICLE_COLOR,		"blue",		DEF_BG, 0},
+     {"author",		AUTHOR_COLOR,		"magenta",	DEF_BG, 0},
+     {"boldtext", 	BOLD_COLOR,		"yellow", 	DEF_BG, SLTT_BOLD_MASK},
+     {"box", 		BOX_COLOR, 		DEF_FG, 	"blue", 0},
+     {"cursor",		CURSOR_COLOR,		"brightgreen",	DEF_BG, SLTT_REV_MASK},
+     {"description",	GROUP_DESCR_COLOR,	"magenta",	DEF_BG, 0},
+     {"error",		ERROR_COLOR,		"red",		DEF_BG, SLTT_BLINK_MASK},
+     {"frame", 		FRAME_COLOR,		"yellow", 	"blue", SLTT_REV_MASK},
+     {"group",		GROUP_COLOR,		"blue",		DEF_BG, 0},
+     {"grouplens_display",GROUPLENS_DISPLAY_COLOR,"blue",	DEF_BG, 0},
+     {"header_name",	SLRN_HEADER_KEYWORD_COLOR,"green", 	DEF_BG, SLTT_BOLD_MASK},
+     {"header_number",	HEADER_NUMBER_COLOR,	"green",	DEF_BG, 0},
+     {"headers",	HEADER_COLOR,		"brightcyan",	DEF_BG, 0},
+     {"high_score",	HIGH_SCORE_COLOR,	"red",		DEF_BG, SLTT_BOLD_MASK},
+     {"italicstext", 	ITALICS_COLOR,		"magenta", 	DEF_BG, 0},
+     {"menu",		MENU_COLOR,		"yellow",	"blue", SLTT_REV_MASK},
+     {"menu_press",	MENU_PRESS_COLOR,	"blue",		"yellow", 0},
+     {"normal",		0,			DEF_FG,		DEF_BG, 0},
+     {"pgpsignature",	PGP_SIGNATURE_COLOR,	"red",		DEF_BG, 0},
+     {"quotes",		QUOTE_COLOR,		"red",		DEF_BG, 0},
+     {"response_char",	RESPONSE_CHAR_COLOR,	"green", 	DEF_BG, SLTT_BOLD_MASK},
+     {"selection", 	SELECT_COLOR,		"yellow", 	"blue", SLTT_BOLD_MASK},
+     {"signature",	SIGNATURE_COLOR,	"red",		DEF_BG, 0},
+     {"status",		STATUS_COLOR,		"yellow",	"blue", SLTT_REV_MASK},
+     {"subject",	SUBJECT_COLOR,		DEF_FG,		DEF_BG, 0},
+     {"thread_number",	THREAD_NUM_COLOR,	"blue",		DEF_BG, SLTT_BOLD_MASK},
+     {"tilde",		SLRN_TILDE_COLOR,	"green",	DEF_BG, SLTT_BOLD_MASK},
+     {"tree",		TREE_COLOR,		"red",		DEF_BG, 0},
+     {"underlinetext", 	UNDERLINE_COLOR,	"magenta", 	DEF_BG, SLTT_BOLD_MASK},
+     {"verbatum", 	VERBATUM_COLOR,		"green", 	DEF_BG, 0},
+
      {NULL, -1, NULL, NULL, 0}
 };
 
@@ -723,7 +755,13 @@ int slrn_set_object_color (char *name, char *fg, char *bg)
 	  }
 	ct++;
      }
-   
+
+   if (0 == strncmp (name, "quotes", 6))
+     {
+	SLtt_set_color (QUOTE_COLOR + atoi (name+6), name, fg, bg);
+	return 0;
+     }
+
    slrn_error ("%s is not a color object", name);
    return -1;
 }
@@ -747,29 +785,36 @@ static int mono_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
    char *what = table->string_args[1];
    char *attr;
    int i;
-   
+   SLtt_Char_Type mono_attr = 0;
    Color_Handle_Type *ct = Color_Handles;
+
+   for (i = 2; i < argc; i++)
+     {
+	attr = table->string_args[i];
+	if (!strcmp (attr, "bold")) mono_attr |= SLTT_BOLD_MASK;
+	else if (!strcmp (attr, "blink")) mono_attr |= SLTT_BLINK_MASK;
+	else if (!strcmp (attr, "underline")) mono_attr |= SLTT_ULINE_MASK;
+	else if (!strcmp (attr, "reverse")) mono_attr |= SLTT_REV_MASK;
+	else if (!strcmp (attr, "none")) mono_attr = 0;
+	else exit_unknown_object ();
+     }
    
    while (ct->name != NULL)
      {
 	if (!strcmp (ct->name, what))
 	  {
-	     SLtt_Char_Type mono_attr = 0;
-	     for (i = 2; i < argc; i++)
-	       {
-		  attr = table->string_args[i];
-		  if (!strcmp (attr, "bold")) mono_attr |= SLTT_BOLD_MASK;
-		  else if (!strcmp (attr, "blink")) mono_attr |= SLTT_BLINK_MASK;
-		  else if (!strcmp (attr, "underline")) mono_attr |= SLTT_ULINE_MASK;
-		  else if (!strcmp (attr, "reverse")) mono_attr |= SLTT_REV_MASK;
-		  else if (!strcmp (attr, "none")) mono_attr = 0;
-		  else exit_unknown_object ();
-	       }
 	     SLtt_set_mono (ct->value, NULL, mono_attr);
 	     return 0;
 	  }
 	ct++;
      }
+
+   if (0 == strncmp (what, "quotes", 6))
+     {
+	SLtt_set_mono (QUOTE_COLOR + atoi (what+6), NULL, mono_attr);
+	return 0;
+     }
+
    exit_unknown_object ();
    return 0;
 }
@@ -806,7 +851,7 @@ User_Info_Variable_Type;
 
 static User_Info_Variable_Type User_Info_Variables[] = /*{{{*/
 {
-     {"hostname", (char **)Slrn_User_Info.host, MAX_HOST_NAME_LEN},
+     {"hostname", &Slrn_User_Info.hostname, 0},
      {"scorefile", &Slrn_Score_File, 0},
 
 #define USER_INFO_OBSOLETE_STARTS_HERE 2
@@ -892,15 +937,39 @@ static Server_List_Type *Server_List;
 
 static Server_List_Type *find_server (char *host) /*{{{*/
 {
-   Server_List_Type *s = Server_List;
-   
-   while (s != NULL)
+   Server_List_Type *s;
+   char *port;
+
+   /* host may contain information about the port in the form "address:port".
+    * First try to find a match with the port information.  If that fails, 
+    * find a match without it.
+    */
+   host = slrn_safe_strmalloc (host);
+   port = slrn_strchr (host, ':');
+
+   while (1)
      {
-	if (0 == slrn_case_strcmp ((unsigned char *)host, 
-				   (unsigned char *) s->host))
+	s = Server_List;
+	while (s != NULL)
+	  {
+	     if (0 == slrn_case_strcmp ((unsigned char *)host, 
+					(unsigned char *) s->host))
+	       break;
+	     s = s->next;
+	  }
+
+	if (s != NULL)
 	  break;
-	s = s->next;
+
+	if (port == NULL)
+	  break;
+	
+	/* Try again without port information */
+	*port = 0;
+	port = NULL;
      }
+
+   slrn_free (host);
    return s;
 }
 
@@ -1003,7 +1072,8 @@ int slrn_get_authorization (char *host, char **name, char **pass) /*{{{*/
 {
    Server_List_Type *s;
    char buf[256];
-   
+   int do_newline = 1;
+
    *name = NULL;
    *pass = NULL;
    
@@ -1018,6 +1088,13 @@ int slrn_get_authorization (char *host, char **name, char **pass) /*{{{*/
    if (*s->username == 0)
      {
 	*buf = 0;
+	if (do_newline)
+	  {
+	     if (0 == (Slrn_TT_Initialized  & SLRN_SMG_INIT))
+	       putc ('\n', stdout);
+	     do_newline = 0;
+	  }
+
 	if (-1 == slrn_read_input ("Username", NULL, buf, 1, 0))
 	  return -1;
 	SLFREE (s->username);
@@ -1027,6 +1104,12 @@ int slrn_get_authorization (char *host, char **name, char **pass) /*{{{*/
    if (*s->password == 0)
      {
 	*buf = 0;
+	if (do_newline)
+	  {
+	     if (0 == (Slrn_TT_Initialized  & SLRN_SMG_INIT))
+	       putc ('\n', stdout);
+	  }
+
 	if (-1 == slrn_read_input_no_echo ("Password", NULL, buf, 1, 0))
 	  return -1;
 	SLFREE (s->password);
@@ -1054,7 +1137,12 @@ static void slrn_init_modes (void) /*{{{*/
 void slrn_startup_initialize (void) /*{{{*/
 {
    Color_Handle_Type *h;
-   
+   int i;
+
+#if SLRN_HAS_SLANG && defined(SLRN_MACRO_DIR)
+   Slrn_Macro_Dir = slrn_safe_strmalloc (SLRN_MACRO_DIR);
+#endif
+
    slrn_init_modes ();
    SLang_init_case_tables ();
    
@@ -1067,16 +1155,25 @@ void slrn_startup_initialize (void) /*{{{*/
 	SLtt_set_mono (h->value, NULL, h->mono);
 	h++;
      }
+   for (i = 1; i < MAX_QUOTE_LEVELS; i++)
+     {
+	SLtt_set_color (QUOTE_COLOR + i, NULL, "red", DEF_BG);
+	SLtt_set_mono (QUOTE_COLOR + i, NULL, 0);
+     }
 
 #if !defined(IBMPC_SYSTEM)
    /* We are not using the blink characters (unless in mono) */
    SLtt_Blink_Mode = 0;
 #endif
-   
+
+   /* The following are required by GNKSA */
+   (void) slrn_set_visible_headers ("From:,Subject:,Newsgroups:,Followup-To:,Reply-To:");
+				       
    (void) slrn_set_header_format (0, "%F%-5S%G%-5l:[%12r]%t%s");
    (void) slrn_set_header_format (1, "%F%G%-5l:[%12r]%t%s");
    (void) slrn_set_header_format (2, "%F%-5l:%t%s");
    (void) slrn_set_header_format (3, "%F%-5S%-5l:%t%50s %r");
+   
 }
 
 
