@@ -1,24 +1,23 @@
 /* -*- mode: C; mode: fold; -*- */
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
+/*
+ This file is part of SLRN.
 
+ Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA. 
+*/
 #include "config.h"
 #include "slrnfeat.h"
 
@@ -138,13 +137,23 @@ static int unpack_as_shell_archive (FILE *fp, char *buf, int size) /*{{{*/
 
 /*}}}*/
 
+/* Note: this function modifies name */
 static FILE *open_output_file (char *name, char *type, int mode, FILE *use_this) /*{{{*/
 {
    FILE *fp;
-   
+   char *p;
+
    if (use_this != NULL)
      return use_this;
    
+   p = name;
+   while (*p != 0)
+     {
+	if (*p == '/')
+	  *p = '_';
+	p++;
+     }
+
    fp = fopen (name, "wb");
 	
    if (fp == NULL)
@@ -816,6 +825,9 @@ static int check_and_uudecode (FILE *fp, FILE *pipe_fp,
 #endif
 		       continue;
 		    }
+		  /* This may still be a bogus line, e.g., a header. */
+		  if (NULL != slrn_strchr (buf, ' '))
+		    continue;
 	       }
 	     
 	     
