@@ -1,25 +1,23 @@
 /* -*- mode: C; mode: fold; -*- */
-/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
- *
- * This file is part of slrn.
- *
- * Slrn is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- * 
- * Slrn is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with Slrn; see the file COPYING.  If not, write to the Free
- * Software Foundation, 59 Temple Place - Suite 330, 
- * Boston, MA  02111-1307, USA.
- */
+/*
+ This file is part of SLRN.
 
+ Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
 
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option)
+ any later version.
+
+ This program is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ more details.
+
+ You should have received a copy of the GNU General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 675
+ Mass Ave, Cambridge, MA 02139, USA. 
+*/
 #include "config.h"
 
 #ifndef SLRNPULL_CODE
@@ -441,10 +439,20 @@ static char *server_read_and_malloc (void) /*{{{*/
    buffer_len_max = buffer_len = 0;
    failed = 0;
    
-   while (NULL != (Slrn_Server_Obj->sv_read_line (line, sizeof(line))))
+   while (1)
      {
 	unsigned int len;
-	
+	int status;
+
+	status = Slrn_Server_Obj->sv_read_line (line, sizeof(line));
+	if (status == -1)
+	  {
+	     slrn_free (mbuf);
+	     return NULL;
+	  }
+	if (status == 0)
+	  break;
+     
 	if (failed) continue;
 	
 	len = strlen (line);
@@ -493,7 +501,7 @@ static int read_head_into_xover (int id, Slrn_XOver_Type *xov) /*{{{*/
 
 int slrn_open_xover (int min, int max) /*{{{*/
 {
-   int id, status;
+   int id, status = -1;
    XOver_Done = 1;
 
    if (Slrn_Server_Obj->sv_has_xover && !Suspend_XOver_For_Kill)
@@ -531,8 +539,8 @@ int slrn_open_xover (int min, int max) /*{{{*/
 	  break;
      }
 
-   if (id > max) 
-     return -1;
+   if (id > max)
+     return status;
 
    XOver_Next = XOver_Min = id;
    XOver_Max = max;
@@ -615,7 +623,6 @@ int slrn_read_xover (Slrn_XOver_Type *xov) /*{{{*/
 {
    char buf [NNTP_BUFFER_SIZE];
    int id;
-   char *overview;
    int status;
    
    if (XOver_Done)
@@ -623,22 +630,24 @@ int slrn_read_xover (Slrn_XOver_Type *xov) /*{{{*/
 
    if (Slrn_Server_Obj->sv_has_xover && !Suspend_XOver_For_Kill)
      {
-	while (NULL != (overview = Slrn_Server_Obj->sv_read_line (buf, sizeof (buf))))
+	while (1)
 	  {
-	     id = atoi (overview);
+	     status = Slrn_Server_Obj->sv_read_line (buf, sizeof (buf));
+	     if (status == -1)
+	       return -1;
+	     if (status == 0)
+	       {
+		  XOver_Done = 1;
+		  return 0;
+	       }
+
+	     id = atoi (buf);
 	     if ((id >= XOver_Min) && (id <= XOver_Max))
 	       break;
-	     
 	     /* else server screwed up and gave bad response.  Ignore it. */
 	  }
 
-	if (overview == NULL)
-	  {
-	     XOver_Done = 1;
-	     return 0;
-	  }
-
-	if (-1 == parse_xover_line (overview, xov))
+	if (-1 == parse_xover_line (buf, xov))
 	  return -1;
 	
 	return 1;
